# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from otcextensions.sdk.rds.v3 import _proxy
from otcextensions.sdk.rds.v3 import backup
from otcextensions.sdk.rds.v3 import configuration
from otcextensions.sdk.rds.v3 import datastore
from otcextensions.sdk.rds.v3 import flavor
from otcextensions.sdk.rds.v3 import instance

from openstack.tests.unit import test_proxy_base


class TestRdsProxy(test_proxy_base.TestProxyBase):
    def setUp(self):
        super(TestRdsProxy, self).setUp()
        self.proxy = _proxy.Proxy(self.session)


class TestFlavor(TestRdsProxy):
    def test_flavors(self):
        self.verify_list(self.proxy.flavors,
                         flavor.Flavor,
                         method_kwargs={
                             'datastore_name': 'MySQL',
                             'version_name': '5.7'
                         },
                         expected_kwargs={
                             'datastore_name': 'MySQL',
                             'version_name': '5.7'
                         })


class TestDatastore(TestRdsProxy):
    def test_datastores(self):
        self.verify_list(
            self.proxy.datastores,
            datastore.Datastore,
            method_args=['ss'],
            expected_kwargs={
                'database_name': 'ss'
            },
            expected_args=[]
        )


class TestConfiguration(TestRdsProxy):
    def test_configurations(self):
        self.verify_list(self.proxy.configurations,
                         configuration.Configuration,
                         expected_kwargs={'paginated': False})

    def test_get_configuration(self):
        self.verify_get(
            self.proxy.get_configuration,
            configuration.Configuration)

    def test_create_configuration(self):
        self.verify_create(
            self.proxy.create_configuration,
            configuration.Configuration,
            method_kwargs={'a': 'b'},
            expected_kwargs={'prepend_key': False, 'a': 'b'})

    def test_delete_configuration(self):
        self.verify_delete(self.proxy.delete_configuration,
                           configuration.Configuration, False)

    def test_delete_configuration_ignore(self):
        self.verify_delete(self.proxy.delete_configuration,
                           configuration.Configuration, True)

    def test_update_configuration(self):
        self.verify_update(self.proxy.update_configuration,
                           configuration.Configuration)

    def test_apply_configuration(self):
        self._verify(
            'otcextensions.sdk.rds.v3.configuration.Configuration.apply',
            self.proxy.apply_configuration,
            method_args=["val", ['a', 'b']],
            expected_args=[self.proxy, ['a', 'b']]
        )


class TestBackup(TestRdsProxy):
    def test_backups(self):
        self.verify_list(
            self.proxy.backups,
            backup.Backup,
            method_args=['inst'],
            expected_args=[],
            expected_kwargs={'instance_id': 'inst'}
        )

    def test_create_backup(self):
        self.verify_create(
            self.proxy.create_backup,
            backup.Backup,
            method_args=['inst'],
            method_kwargs={'x': 1, 'y': 2, 'z': 3},
            expected_kwargs={
                'x': 1, 'y': 2, 'z': 3,
                'instance_id': 'inst'
            },
            expected_args=[]
        )

    def test_delete_backup(self):
        self.verify_delete(self.proxy.delete_backup,
                           backup.Backup, False)

    def test_delete_backup_ignore(self):
        self.verify_delete(self.proxy.delete_backup,
                           backup.Backup, True)

    def test_find_backup(self):
        self.verify_find(
            self.proxy.find_backup,
            backup.Backup,
            method_args=['name'],
            expected_kwargs={
                'ignore_missing': True,
                'instance_id': 'name'
            },
            expected_args=['resource_name']
        )

    def test_wait_for_backup(self):
        value = backup.Backup(id='fake')
        self.verify_wait_for_status(
            self.proxy.wait_for_backup,
            method_args=[value],
            expected_args=[
                self.proxy,
                value,
                'COMPLETED',
                ['FAILED'],
                2, 300
            ])

    def test_download_links(self):
        self.verify_list(
            self.proxy.backup_download_links,
            backup.BackupFile,
            method_args=['bck_id'],
            expected_kwargs={'backup_id': 'bck_id'},
            expected_args=[]
        )


class TestInstance(TestRdsProxy):
    def test_instances(self):
        self.verify_list(self.proxy.instances,
                         instance.Instance)

    def test_create_instance(self):
        self.verify_create(
            self.proxy.create_instance,
            instance.Instance)

    def test_get_instance(self):
        self.verify_get(
            self.proxy.get_instance,
            instance.Instance)

    def test_find_instance(self):
        self.verify_find(
            self.proxy.find_instance,
            instance.Instance)

    def test_delete_instance(self):
        self.verify_delete(self.proxy.delete_instance,
                           instance.Instance, False)

    def test_delete_instance_ignore(self):
        self.verify_delete(self.proxy.delete_instance,
                           instance.Instance, True)

    def test_fetch_restore_times(self):
        self._verify(
            'otcextensions.sdk.rds.v3.instance.Instance.fetch_restore_times',
            self.proxy.get_instance_restore_time,
            method_args=["inst"],
            expected_args=[self.proxy]
        )

    def test_restore(self):
        self._verify(
            'otcextensions.sdk.rds.v3.instance.Instance.restore',
            self.proxy.restore_instance,
            method_args=["inst"],
            method_kwargs={
                'backup': backup.Backup(id='bck'),
                'restore_time': 'rt'},
            expected_args=[self.proxy, backup.Backup(id='bck'), 'rt'],
        )

    def test_get_instance_configuration(self):
        pass

    def test_update_instance_configuration(self):
        pass

    def test_get_instance_backup_policy(self):
        self._verify(
            'otcextensions.sdk.rds.v3.instance.Instance.get_backup_policy',
            self.proxy.get_instance_backup_policy,
            method_args=["val"],
            expected_args=[self.proxy]
        )

    def test_update_instance_backup_policy(self):
        self._verify(
            'otcextensions.sdk.rds.v3.instance.Instance.set_backup_policy',
            self.proxy.set_instance_backup_policy,
            method_args=["val"],
            method_kwargs={
                'keep_days': 1,
                'start_time': '2',
                'period': '3'
            },
            expected_kwargs={
                'keep_days': 1,
                'start_time': '2',
                'period': '3'
            },
            expected_args=[self.proxy]
        )

    def test_restart_instance(self):
        self._verify(
            'otcextensions.sdk.rds.v3.instance.Instance.restart',
            self.proxy.restart_instance,
            method_args=["val"],
            expected_args=[self.proxy]
        )

    def test_enlarge_instance_volume(self):
        self._verify(
            'otcextensions.sdk.rds.v3.instance.Instance.enlarge_volume',
            self.proxy.enlarge_instance_volume,
            method_args=["val"],
            method_kwargs={
                'size': 200
            },
            expected_args=[self.proxy, 200]
        )

    def test_update_flavor(self):
        self._verify(
            'otcextensions.sdk.rds.v3.instance.Instance.update_flavor',
            self.proxy.change_instance_flavor,
            method_args=["val"],
            method_kwargs={
                'spec_code': 'test.spec.code'
            },
            expected_args=[self.proxy, 'test.spec.code']
        )

    def test_get_instance_logs(self):
        self._verify(
            'otcextensions.sdk.rds.v3.instance.Instance.get_logs',
            self.proxy.get_instance_logs,
            method_args=["val"],
            method_kwargs={
                'log_type': 'errorlog',
                'start_date': '2020-01-01T12:34:56+0000',
                'end_date': '2020-01-02T12:34:56+0000',
                'offset': 10,
                'limit': 20,
                'level': 'ERROR'
            },
            expected_args=[self.proxy, 'errorlog', '2020-01-01T12:34:56+0000',
                           '2020-01-02T12:34:56+0000', 10, 20, 'ERROR']
        )

    def test_add_tag(self):
        self._verify(
            'otcextensions.sdk.rds.v3.instance.Instance.add_tag',
            self.proxy.add_tag,
            method_args=["val"],
            method_kwargs={
                'key': 'tagkey',
                'value': 'tagval'
            },
            expected_args=[self.proxy, 'tagkey', 'tagval']
        )

    def test_remove_tag(self):
        self._verify(
            'otcextensions.sdk.rds.v3.instance.Instance.remove_tag',
            self.proxy.remove_tag,
            method_args=["val"],
            method_kwargs={
                'key': 'tagkey'
            },
            expected_args=[self.proxy, 'tagkey']
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.tests.unit import base
from otcextensions.sdk.apig.v2 import signature

EXAMPLE = {
    "name": "signature_demo",
    "sign_key": "signkeysignkey",
    "sign_secret": "sig************ret"
}


class TestThrottlingPolicy(base.TestCase):

    def test_basic(self):
        sot = signature.Signature()
        self.assertEqual(
            '/apigw/instances/%(gateway_id)s/signs',
            sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertEqual('signs', sot.resources_key)

    def test_make_it(self):
        sot = signature.Signature(**EXAMPLE)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['sign_key'], sot.sign_key)
        self.assertEqual(EXAMPLE['sign_secret'], sot.sign_secret)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
import mock

from otcextensions.osclient.cbr.v3 import task
from otcextensions.tests.unit.osclient.cbr.v3 import fakes


class TestTask(fakes.TestCBR):

    def setUp(self):
        super(TestTask, self).setUp()

    def test_flatten(self):
        obj = fakes.FakeTask.create_one()

        flat_data = task._flatten_task(obj)

        data = (
            flat_data['id'],
            flat_data['checkpoint_id'],
            flat_data['policy_id'],
            flat_data['provider_id'],
            flat_data['vault_id'],
            flat_data['vault_name'],
            flat_data['operation_type'],
            flat_data['error_mesage'],
            flat_data['error_code'],
            flat_data['created_at'],
            flat_data['ended_at'],
            flat_data['started_at'],
            flat_data['updated_at']
        )

        cmp_data = (
            obj.id,
            obj.checkpoint_id,
            obj.policy_id,
            obj.provider_id,
            obj.vault_id,
            obj.vault_name,
            obj.operation_type,
            obj.error_info.message,
            obj.error_info.code,
            obj.created_at,
            obj.ended_at,
            obj.started_at,
            obj.updated_at,
        )

        self.assertEqual(data, cmp_data)


class TestListTask(fakes.TestCBR):

    objects = fakes.FakeTask.create_multiple(3)

    columns = ('id', 'checkpoint_id', 'provider_id',
               'operation_type', 'created_at', 'ended_at')

    data = []

    for s in objects:
        flat_data = task._flatten_task(s)
        data.append((
            flat_data['id'],
            flat_data['checkpoint_id'],
            flat_data['provider_id'],
            flat_data['operation_type'],
            flat_data['created_at'],
            flat_data['ended_at']
        ))

    def setUp(self):
        super(TestListTask, self).setUp()

        self.cmd = task.ListTasks(self.app, None)

        self.client.tasks = mock.Mock()
        self.client.api_mock = self.client.tasks

    def test_default(self):
        arglist = []

        verifylist = []

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [
            self.objects
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_once_with()

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))


class TestShowTask(fakes.TestCBR):

    object = fakes.FakeTask.create_one()

    columns = (
        'id',
        'checkpoint_id',
        'policy_id',
        'provider_id',
        'vault_id',
        'vault_name',
        'operation_type',
        'error_mesage',
        'error_code',
        'created_at',
        'ended_at',
        'started_at',
        'updated_at'
    )

    flat_data = task._flatten_task(object)

    data = (
        flat_data['id'],
        flat_data['checkpoint_id'],
        flat_data['policy_id'],
        flat_data['provider_id'],
        flat_data['vault_id'],
        flat_data['vault_name'],
        flat_data['operation_type'],
        flat_data['error_mesage'],
        flat_data['error_code'],
        flat_data['created_at'],
        flat_data['ended_at'],
        flat_data['started_at'],
        flat_data['updated_at']
    )

    def setUp(self):
        super(TestShowTask, self).setUp()

        self.cmd = task.ShowTask(self.app, None)

        self.client.get_task = mock.Mock()

    def test_default(self):
        arglist = [
            'task'
        ]
        verifylist = [
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.get_task.side_effect = [
            self.object
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.get_task.assert_called_once_with('task')

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
import json

from openstack import exceptions
from openstack import resource


class NodeSpec(resource.Resource):
    #: Node architecture.
    arch = resource.Body("arch")
    #: Number of CPUs.
    cpu = resource.Body("cpu", type=int)
    #: Creation time, in the format of YYYY-MM-DDThh:mm:ss (UTC).
    created_at = resource.Body("created_at")
    #: Number of application instances deployed on a node.
    deployment_num = resource.Body("deployment_num", type=int)
    #: Description.
    description = resource.Body("description")
    #: Whether to enable the GPU.
    enable_gpu = resource.Body("enable_gpu")
    #: Number of GPUs.
    gpu_num = resource.Body("gpu_num", type=int)
    #: Host IP of a node.
    host_ips = resource.Body("host_ips", type=list)
    #: Host name of a node.
    host_name = resource.Body("host_name")
    #: Node ID.
    id = resource.Body("id")
    #: Status of a model instance on a node.
    instance_status = resource.Body("instance_status")
    #: Memory size, in MB.
    memory = resource.Body("memory", type=int)
    #: Node name.
    name = resource.Body("name")
    #: Node label.
    node_label = resource.Body("node_label")
    #: OS name.
    os_name = resource.Body("os_name")
    #: OS type.
    os_type = resource.Body("os_type")
    #: OS version.
    os_version = resource.Body("os_version")
    #: Inference URL of a node.
    predict_url = resource.Body("predict_url")
    #: Host status.
    state = resource.Body("state")
    #: Update time, in the format of YYYY-MM-DDThh:mm:ss (UTC).
    updated_at = resource.Body("updated_at")


class CustomSpec(resource.Resource):
    #: Number of required CPUs.
    cpu = resource.Body("cpu", type=float)
    #: Number of GPUs, which can be decimals.
    gpu_p4 = resource.Body("gpu_p4", type=float)
    #: Required memory capacity, in MB.
    memory = resource.Body("memory", type=int)


class ScheduleSpec(resource.Resource):
    #: Value that maps to the time unit.
    duration = resource.Body("duration", type=int)
    #: Scheduling time unit.
    time_unit = resource.Body("time_unit")
    #: Scheduling type.
    type = resource.Body("type")


class ConfigSpec(resource.Resource):
    #: Additional model deployment attribute.
    additional_properties = resource.Body("additional_properties", type=dict)
    #: ID of a dedicated resource pool.
    cluster_id = resource.Body("cluster_id")
    #: Custom specifications.
    custom_spec = resource.Body("custom_spec", type=CustomSpec)
    #: OBS path of the output data of a batch job.
    dest_path = resource.Body("dest_path")
    #: Environment variable key-value pair required for running a model.
    envs = resource.Body("envs", type=dict)
    #: Task finished time, in milliseconds calculated from 1970.
    finished_time = resource.Body("finished_time", type=int)
    #: Number of instances deployed in a model.
    instance_count = resource.Body("instance_count", type=int)
    #: Mapping between input parameters and CSV data.
    mapping_rule = resource.Body("mapping_rule", type=dict)
    #: Mapping type of the input data.
    mapping_type = resource.Body("mapping_type")
    #: Model ID.
    model_id = resource.Body("model_id")
    #: Model name.
    model_name = resource.Body("model_name")
    #: Model version.
    model_version = resource.Body("model_version")
    #: Inference path of a batch job.
    request_uri = resource.Body("req_uri")
    #: Whether auto scaling is enabled.
    scaling = resource.Body("scaling", type=bool)
    #: Model source.
    source_type = resource.Body("source_type")
    #: Resource flavor.
    specification = resource.Body("specification")
    #: OBS path of the input data of a batch job.
    src_path = resource.Body("src_path")
    #: Data source type.
    src_type = resource.Body("src_type")
    #: Task start time, in milliseconds calculated from 1970.
    start_time = resource.Body("start_time", type=int)
    #: Model status.
    status = resource.Body("status")
    #: Whether a model supports online debugging.
    support_debug = resource.Body("support_debug", type=bool)
    #: Traffic weight allocated to a model.
    weight = resource.Body("weight", type=int)


class Service(resource.Resource):
    base_path = "/services"
    resources_key = "services"

    # capabilities
    allow_create = True
    allow_list = True
    allow_commit = True
    allow_delete = True
    allow_fetch = True

    _query_mapping = resource.QueryParameters(
        "cluster_id",
        "infer_type",
        "limit",
        "model_id",
        "name",
        "offset",
        "order",
        "service_id",
        "sort_by",
        "status",
        "workspace_id",
        name="service_name",
    )

    # Properties
    #: Access address of an inference request. This parameter is
    #:  returned when infer_type is set to real-time.
    access_address = resource.Body("access_address")
    #: Additional service attribute, which facilitates service management.
    additional_properties = resource.Body("additional_properties", type=dict)
    #: Request address of the user-defined domain name. This
    #:  parameter is returned after the domain name is bound.
    bind_access_address = resource.Body("bind_access_address")
    #: ID of the dedicated resource pool used by the real-time or batch
    #:  service.
    cluster_id = resource.Body("cluster_id")
    #: Service configuration (If a service is shared, only model_id,
    #:  model_name, and model_version are returned.
    config = resource.Body("config", type=list, list_type=ConfigSpec)
    #: Online debugging address of a real-time service.
    debug_url = resource.Body("debug_url")
    #: Service description, which contains a maximum of 100 characters.
    description = resource.Body("description")
    #: Time when a real-time service automatically stops,
    #:  in milliseconds calculated from 1970.1.1 0:0:0 UTC.
    due_time = resource.Body("due_time", type=int)
    #: Error message. When status is failed, an error message
    #:  carrying the failure cause is returned.
    error_msg = resource.Body("error_msg")
    #: Number of failed service calls.
    failed_times = resource.Body("failed_times", type=int)
    #: Inference mode. The value can be real-time, batch.
    infer_type = resource.Body("infer_type")
    #: Total number of service calls.
    invocation_times = resource.Body("invocation_times", type=int)
    #: Whether a free-of-charge flavor is used.
    is_free = resource.Body("is_free", type=bool)
    #: Whether a service is subscribed.
    is_shared = resource.Body("is_shared", type=bool)
    #: Service Name.
    name = resource.Body("name", alias="service_name")
    #: Network ID.
    network_id = resource.Body("subnet_network_id")
    #: Operation time of a request.
    operation_time = resource.Body("operation_time", type=int)
    #: User to which a service belongs.
    owner = resource.Body("owner")
    #: Deployment progress.
    progress = resource.Body("progress", type=int)
    #: Project to which a service belongs.
    project = resource.Body("project")
    #: Latest service release time, in milliseconds
    #:  calculated from 1970.1.1 0:0:0 UTC.
    publish_at = resource.Body("publish_at", type=int)
    #: Resource ID array for the resource IDs generated by the target model.
    resource_ids = resource.Body("resource_ids", type=list)
    #: ID of the VPC to which a real-time service instance is deployed.
    router_id = resource.Body("vpc_id")
    #: Service scheduling configuration, which can be configured only
    #:  for real-time services.
    schedule = resource.Body("schedule", type=list, list_type=ScheduleSpec)
    #: Security group.
    security_group_id = resource.Body("security_group_id")
    #: Service ID.
    service_id = resource.Body("service_id", alternate_id=True)
    #: Service name.
    service_name = resource.Body("service_name")
    #: Number of subscribed services.
    shared_count = resource.Body("shared_count", type=int)
    #: Service status, which can be running, deploying, concerning, failed,
    #:  stopped, or finished.
    status = resource.Body("status")
    #: Tenant to which a service belongs.
    tenant = resource.Body("tenant")
    #: Time when the service status changes.
    transition_at = resource.Body("transition_at", type=int)
    #: Time when the configuration used by a service is updated,
    #:  in milliseconds calculated from 1970.1.1 0:0:0 UTC.
    update_time = resource.Body("update_time", type=int)
    #: ID of the workspace to which a service belongs.
    workspace_id = resource.Body("workspace_id")

    def _translate_response(self, response, has_body=None, error_message=None,
                            resource_response_key=None):
        """Given a KSA response, inflate this instance with its data
        DELETE operations don't return a body, so only try to work
        with a body when has_body is True.
        This method updates attributes that correspond to headers
        and body on this instance and clears the dirty set.
        """
        if has_body is None:
            has_body = self.has_body

        exceptions.raise_from_response(response, error_message=error_message)

        if has_body:
            try:
                body = response.json()
                if self.resource_key and self.resource_key in body:
                    body = body[self.resource_key]

                for key in (
                    "model_metrics",
                    "config",
                    "apis",
                    "output_params",
                    "input_params",
                ):
                    if key in body:
                        if isinstance(body[key], str):
                            body[key] = json.loads(body[key])
                        elif key in (
                            "output_params",
                            "input_params",
                        ) and isinstance(body[key], list):
                            for param in body[key]:
                                if isinstance(param.get("param_desc"), str):
                                    param["param_desc"] = json.loads(
                                        param["param_desc"]
                                    )

                # Do not allow keys called "self" through. Glance chose
                # to name a key "self", so we need to pop it out because
                # we can't send it through cls.existing and into the
                # Resource initializer. "self" is already the first
                # argument and is practically a reserved word.
                body.pop("self", None)

                body_attrs = self._consume_body_attrs(body)
                if self._allow_unknown_attrs_in_body:
                    body_attrs.update(body)
                    self._unknown_attrs_in_body.update(body)
                elif self._store_unknown_attrs_as_properties:
                    body_attrs = self._pack_attrs_under_properties(
                        body_attrs, body
                    )

                self._body.attributes.update(body_attrs)
                self._body.clean()
                if self.commit_jsonpatch or self.allow_patch:
                    # We need the original body to compare against
                    self._original_body = body_attrs.copy()
            except ValueError:
                # Server returned not parse-able response (202, 204, etc)
                # Do simply nothing
                pass

        headers = self._consume_header_attrs(response.headers)
        self._header.attributes.update(headers)
        self._header.clean()
        self._update_location()
        dict.update(self, self.to_dict())



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.unit import base

from otcextensions.sdk.dds.v3 import recycle_instance

EXAMPLE = {
    "total_count": 1,
    "instances": [
        {
            "id": "07fc12a8e0e94df7a3fcf53d0b5e1605in02",
            "name": "test1",
            "mode": "ReplicaSet",
            "datastore": {
                "type": "DDS-Community",
                "version": "4.0"
            },
            "pay_mode": "0",
            "enterprise_project_id": "0",
            "backup_id": "bf9ee62a7f7044c583c6765c916c36edbr02",
            "created_at": "2022-01-01T10:00:00",
            "deleted_at": "2022-02-01T10:00:00",
            "retained_until": "2022-05-01T10:00:00",
            "status": "Active"
        }
    ]
}


class TestRecycleInstance(base.TestCase):
    def test_basic(self):
        sot = recycle_instance.RecycleInstance()
        path = '/recycle-instances'
        self.assertEqual(path, sot.base_path)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = recycle_instance.RecycleInstance(**EXAMPLE)
        self.assertEqual(EXAMPLE['total_count'], sot.total_count)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource

from otcextensions.common import format


class Trace(resource.Resource):

    base_path = '/%(tracker_name)s/trace'
    resources_key = 'traces'
    service_expectes_json_type = True

    allow_list = True

    # NOTE: resource_type query param need to be renamed, due to a conflict
    # with proxy._list function
    _query_mapping = resource.QueryParameters(
        'service_type', 'res_type', 'res_id', 'res_name',
        'trace_name', 'limit', 'next', 'from', 'to',
        'trace_id', 'level', 'user',
        level='trace_rating',
        res_type='resource_type',
        res_id='resource_id',
        res_name='resource_name')

    # Properties
    tracker_name = resource.URI('tracker_name')
    #: API version
    api_version = resource.Body('api_version')
    #: trace http return code
    code = resource.Body('code')
    #: Trace ID
    id = resource.Body('trace_id', alternate_id=True)
    #: rating of the trace, normal, warning, incident
    level = resource.Body('trace_rating')
    #: remark of the trace
    message = resource.Body('message')
    #: metadata of the trace
    #: *Type: dict*
    meta_data = resource.Body('meta_data', type=dict)
    #: name of the trace
    name = resource.Body('trace_name')
    #: record time stampt
    record_time = resource.Body('record_time', type=format.TimeTMsStr)
    #: trace request content
    request = resource.Body('request')
    #: trace resource id
    resource_id = resource.Body('resource_id')
    #: resource name of the trace
    resource_name = resource.Body('resource_name')
    #: trace resource type
    resource_type = resource.Body('resource_type')
    #: trace response content
    response = resource.Body('response')
    #: trace service type
    service_type = resource.Body('service_type')
    #: user ip of the trace
    source_ip = resource.Body('source_ip')
    #: *Type: int*
    time = resource.Body('time', type=format.TimeTMsStr)
    #: trace source type
    type = resource.Body('trace_type')
    #: trace user information
    user = resource.Body('user')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource


class Snat(resource.Resource):
    resources_key = 'snat_rules'
    resource_key = 'snat_rule'
    base_path = '/snat_rules'

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_delete = True
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'admin_state_up', 'cidr', 'created_at', 'floating_ip_address',
        'floating_ip_id', 'id', 'limit', 'nat_gateway_id', 'network_id',
        'source_type', 'status', 'project_id', project_id='tenant_id'
    )

    # Properties
    #: Specifies the status of the SNAT rule
    admin_state_up = resource.Body('admin_state_up', type=bool)
    #: Specifies a subset of the VPC subnet CIDR block or a
    #: CIDR block of Direct Connect connection.
    cidr = resource.Body('cidr')
    #: Specifies when the rule is created.
    #: The format is yyyy-mm-dd hh:mm:ss.
    created_at = resource.Body('created_at')
    #: Specifies the EIP
    #: Multiple EIPs are separated using commas
    floating_ip_address = resource.Body('floating_ip_address')
    #: Specifies the EIP ID
    #: Multiple EIPs are separated using commas
    floating_ip_id = resource.Body('floating_ip_id')
    #: Specifies the ID of the SNAT rule.
    id = resource.Body('id')
    #: Specifies the gateway ID.
    nat_gateway_id = resource.Body('nat_gateway_id')
    #: Specifies the network ID
    network_id = resource.Body('network_id')
    #: Specifies the project ID.
    project_id = resource.Body('tenant_id')
    #: *0:* Either network_id or cidr can be specified in VPC
    #: *1:* only cidr can be specified over a Direct Connect connection
    #: Default: 0
    source_type = resource.Body('source_type', type=int)
    #: Specifies whether SNAT rule is enabled / disabled
    #: *true:* SNAT rule is enabled
    #: *false:* SNAT rule is disabled
    status = resource.Body('status')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
from openstack.tests.unit import base
from otcextensions.sdk.modelartsv2.v2 import dataset_export_task
from otcextensions.tests.unit.sdk.modelartsv2.v2 import examples
from otcextensions.tests.unit.sdk.utils import assert_attributes_equal

EXAMPLE = examples.DATASET_EXPORT_TASK


class TestDatasetExportTask(base.TestCase):
    def setUp(self):
        super(TestDatasetExportTask, self).setUp()

    def test_basic(self):
        sot = dataset_export_task.DatasetExportTask()

        self.assertEqual(
            "/datasets/%(uri_dataset_id)s/export-tasks", sot.base_path
        )
        self.assertEqual(None, sot.resource_key)
        self.assertEqual("export_tasks", sot.resources_key)

        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_create)
        self.assertFalse(sot.allow_delete)
        self.assertFalse(sot.allow_commit)

        self.assertDictEqual(
            {
                "export_type": "export_type",
                "limit": "limit",
                "marker": "marker",
                "offset": "offset",
            },
            sot._query_mapping._mapping,
        )

    def test_make_it(self):
        sot = dataset_export_task.DatasetExportTask(**EXAMPLE)
        for key, value in EXAMPLE.items():
            if key == 'task_id':
                self.assertEqual(sot.id, value)
            elif key == 'update_time':
                self.assertEqual(sot.updated_at, value)
            elif key == 'create_time':
                self.assertEqual(sot.created_at, value)
            else:
                assert_attributes_equal(self, getattr(sot, key), value)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
import random
import uuid

import mock

from openstackclient.tests.unit import utils

from otcextensions.tests.unit.osclient import test_base

from otcextensions.sdk.rds.v3 import configuration
from otcextensions.sdk.rds.v3 import datastore
from otcextensions.sdk.rds.v3 import flavor
from otcextensions.sdk.rds.v3 import instance
from otcextensions.sdk.rds.v3 import backup


def gen_data(data, columns):
    """Fill expected data tuple based on columns list
    """
    return tuple(getattr(data, attr, '') for attr in columns)


def gen_data_dict(data, columns):
    """Fill expected data tuple based on columns list
    """
    return tuple(data.get(attr, '') for attr in columns)


class TestRds(utils.TestCommand):
    def setUp(self):
        super(TestRds, self).setUp()

        self.app.client_manager.rds = mock.Mock()
        self.app.client_manager.sdk_connection = mock.Mock()

        self.client = self.app.client_manager.rds
        self.sdk_client = self.app.client_manager.sdk_connection

        self.datastore_mock = FakeDatastore
        self.flavor_mock = FakeFlavor
        self.instance_mock = FakeInstance
        self.backup_mock = FakeBackup
        self.configuration_mock = FakeConfiguration


class FakeDatastore(test_base.Fake):
    """Fake one or more datastore versions."""
    @classmethod
    def generate(cls):
        """Create a fake datastore.

        :return:
            A FakeResource object, with id, name, metadata, and so on
        """
        # Set default attributes.
        object_info = {
            'id': 'id-' + uuid.uuid4().hex,
            'name': 'name-' + uuid.uuid4().hex
        }

        return datastore.Datastore(**object_info)


class FakeFlavor(test_base.Fake):
    """Fake one or more VBS Policy"""
    @classmethod
    def generate(cls):
        object_info = {
            'instance_mode': random.choice(['ha', 'replica', 'single']),
            'vcpus': str(random.randint(1, 100)),
            'spec_code': uuid.uuid4().hex,
            'ram': random.randint(1, 10280),
        }
        obj = flavor.Flavor.existing(**object_info)
        return obj


class FakeConfiguration(test_base.Fake):
    """Fake one or more Configuration."""

    @classmethod
    def generate(cls):
        """Create a fake Configuration.

        :param Dictionary attrs:
            A dictionary with all attributes
        :param Dictionary methods:
            A dictionary with all methods
        :return:
            A FakeResource object, with id, name, metadata, and so on
        """
        object_info = {
            'id': 'id-' + uuid.uuid4().hex,
            'name': 'name-' + uuid.uuid4().hex,
            'description': 'descriptions-' + uuid.uuid4().hex,
            'datastore_name': uuid.uuid4().hex,
            'datastore_version_name': uuid.uuid4().hex,
            'values': {},
            'configuration_parameters':
                list({
                    'name': 'name-' + uuid.uuid4().hex,
                    'value': 'value-' + uuid.uuid4().hex,
                    'restart_required': bool(random.getrandbits(1)),
                    'readonly': bool(random.getrandbits(1)),
                    'value_range': uuid.uuid4().hex,
                    'type': 'type-' + uuid.uuid4().hex,
                    'description': 'descr-' + uuid.uuid4().hex

                } for i in range(3)),
            'is_user_defined': bool(random.getrandbits(1)),
            'apply_results': [
                {
                    "instance_id": uuid.uuid4().hex,
                    "instance_name": uuid.uuid4().hex,
                    "restart_required": bool(random.getrandbits(1)),
                    "success": bool(random.getrandbits(1))
                }
            ]

        }

        obj = configuration.Configuration.existing(**object_info)
        return obj


class FakeInstance(test_base.Fake):
    """Fake one or more Instance."""
    @classmethod
    def generate(cls):
        """Create a fake Configuration.

        :return:
            A FakeResource object, with id, name, metadata, and so on
        """

        # Set default attributes.
        object_info = {
            'id': 'id-' + uuid.uuid4().hex,
            'name': 'name-' + uuid.uuid4().hex,
            'status': 'status-' + uuid.uuid4().hex,
            'datastore': {
                'type': 'datastore-' + uuid.uuid4().hex,
                'version': 'version-' + uuid.uuid4().hex,
            },
            'flavor_ref': {
                'id': uuid.uuid4().hex
            },
            'volume': {
                'type': 'type' + uuid.uuid4().hex,
                'size': random.randint(1, 10280),
            },
            'region': 'region' + uuid.uuid4().hex,
            'job_id': 'jobid' + uuid.uuid4().hex
        }

        return instance.Instance(**object_info)


class FakeBackup(test_base.Fake):
    """Fake one or more Backup"""
    @classmethod
    def generate(cls):
        object_info = {
            'id': 'id-' + uuid.uuid4().hex,
            'name': 'name-' + uuid.uuid4().hex,
            'description': uuid.uuid4().hex,
            'datastore': {
                'type': 'datastore-' + uuid.uuid4().hex,
                'version': 'version-' + uuid.uuid4().hex,
            },
            'instance_id': 'instance_id-' + uuid.uuid4().hex,
            'size': random.randint(0, 100),
            'status':
            random.choice(['BUILDING', 'COMPLETED', 'FAILED', 'DELETING']),
            'type': random.choice(['auto', 'manual']),
            'begin_time': uuid.uuid4().hex,
            'end_time': uuid.uuid4().hex
        }
        obj = backup.Backup.existing(**object_info)
        return obj


class FakeBackupFile(test_base.Fake):
    """Fake one or more BackupFile"""
    @classmethod
    def generate(cls):
        object_info = {
            'size': random.randint(1, 100000),
            'name': 'name-' + uuid.uuid4().hex,
            'download_link': uuid.uuid4().hex,
            'expires_at': 'expires-' + uuid.uuid4().hex
        }
        obj = backup.BackupFile.existing(**object_info)
        return obj



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid
from keystoneauth1 import adapter
import mock
from openstack.tests.unit import base

from otcextensions.sdk.sdrs.v1 import task_center as _task_center


EXAMPLE = {
    "job_status": "FAIL",
    "resource_id": uuid.uuid4(),
    "resource_name": "Replication-Pair-4fb5",
    "resource_type": "replications",
    "failure_status": "expandFail",
    "job_id": uuid.uuid4(),
    "job_type": "expandReplicationPairNew",
    "begin_time": "2022-03-18T13:55:33.570Z",
    "error_code": "SDRS.1820",
    "fail_reason": "Failed to expand the capacity"
}


class TestTaskCenter(base.TestCase):

    def setUp(self):
        super(TestTaskCenter, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.get = mock.Mock()
        self.sess.default_microversion = '1'
        self.sess._get_connection = mock.Mock(return_value=self.cloud)
        self.sot = _task_center.FailedTask()

    def test_basic(self):
        sot = _task_center.FailedTask()
        self.assertEqual('', sot.resource_key)
        self.assertEqual('failure_jobs', sot.resources_key)
        self.assertEqual('/task-center/failure-jobs',
                         sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertFalse(sot.allow_create)
        self.assertFalse(sot.allow_fetch)
        self.assertFalse(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertDictEqual({
            'failure_status': 'failure_status',
            'limit': 'limit',
            'marker': 'marker',
            'offset': 'offset',
            'resource_name': 'resource_name',
            'resource_type': 'resource_type',
            'server_group_id': 'server_group_id'
        }, sot._query_mapping._mapping)

    def test_make_it(self):
        test_failed_task = _task_center.FailedTask(**EXAMPLE)
        self.assertEqual(
            EXAMPLE['job_status'],
            test_failed_task.job_status)
        self.assertEqual(
            EXAMPLE['resource_id'],
            test_failed_task.resource_id)
        self.assertEqual(
            EXAMPLE['resource_name'],
            test_failed_task.resource_name)
        self.assertEqual(
            EXAMPLE['resource_type'],
            test_failed_task.resource_type)
        self.assertEqual(
            EXAMPLE['failure_status'],
            test_failed_task.failure_status)
        self.assertEqual(
            EXAMPLE['job_id'],
            test_failed_task.job_id)
        self.assertEqual(
            EXAMPLE['job_type'],
            test_failed_task.job_type)
        self.assertEqual(
            EXAMPLE['begin_time'],
            test_failed_task.begin_time)
        self.assertEqual(
            EXAMPLE['error_code'],
            test_failed_task.error_code)
        self.assertEqual(
            EXAMPLE['fail_reason'],
            test_failed_task.fail_reason)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource
from openstack import utils


class SchedulePolicy(resource.Resource):
    #: whether keep the first backup of current month
    remain_first_backup_of_curMonth = resource.Body(
        "remain_first_backup_of_curMonth")
    #: the max backup amount, min value is 2
    rentention_num = resource.Body("rentention_num", type=int)
    #: backup period, valid values, 1..14 (days)
    frequency = resource.Body("frequency", type=int)
    #: backup start time of every day, example: 12:00
    start_time = resource.Body("start_time")
    #: backup policy status, ``ON``, ``OFF``
    status = resource.Body("status")


class BackupPolicy(resource.Resource):
    """Volume BackupPolicy"""
    resources_key = "backup_policies"
    base_path = "/backuppolicy"

    # capabilities
    allow_create = True
    allow_delete = True
    allow_list = True
    allow_commit = True

    #: Properties
    #: Backup Policy id
    id = resource.Body("backup_policy_id", alternate_id=True)
    #: Backup Policy name
    name = resource.Body("backup_policy_name")
    #: Backup Policy resource count
    policy_resource_count = resource.Body("policy_resource_count")
    #: Backup Policy schedule detail
    scheduled_policy = resource.Body("scheduled_policy", type=SchedulePolicy)
    #: tags
    tags = resource.Body("tags", type=list)

    def execute(self, session):
        """execute backup policy immediately

        :param session: openstack session
        :return: request response
        """
        url = utils.urljoin(self.base_path, self.id, "action")
        return session.post(url, json=None)


class BackupPolicyAssociatedResource(resource.Resource):
    #: Properties
    resource_id = resource.Body("resource_id")
    resource_type = resource.Body("resource_type")
    availability_zone = resource.Body("availability_zone")
    os_vol_host_attr = resource.Body("os_vol_host_attr")
    message = resource.Body("message")
    code = resource.Body("code")


class BackupPolicyResource(resource.Resource):
    base_path = "/backuppolicyresources"

    # capabilities
    allow_create = True

    #: Properties
    # backup_policy_id = resource.Body('backup_policy_id')
    resources = resource.Body(
        'resources', type=list, list_type=BackupPolicyAssociatedResource)
    fail_resources = resource.Body(
        "fail_resources",
        type=list, list_type=BackupPolicyAssociatedResource)
    success_resources = resource.Body(
        "success_resources",
        type=list, list_type=BackupPolicyAssociatedResource)

    def _process(self, session, backup_policy_id, link=True, resources=[]):
        """Link or unlink resources to/from BackupPolicy
        """
        _resources = [dict(resource_id=volume_id, resource_type="volume")
                      for volume_id in resources]
        body = {
            "resources": _resources
        }
        if link:
            body['backup_policy_id'] = backup_policy_id
            uri = self.base_path
        else:
            uri = utils.urljoin(
                self.base_path, backup_policy_id, "deleted_resources")

        response = session.post(uri, json=body)

        fail_resources = []
        success_resources = []

        response_json = response.json()
        for type in ['success_resources', 'fail_resources']:
            if type in response_json:
                for res in response_json[type]:
                    assocResource = \
                        BackupPolicyAssociatedResource.existing(**res)
                    if assocResource.code:
                        fail_resources.append(assocResource)
                    else:
                        success_resources.append(assocResource)
        self._update(
            success_resources=success_resources,
            fail_resources=fail_resources
        )

        return self

    def link(self, session, backup_policy_id, resources):
        """link resources to backup policy

        :param session: openstack session
        :param backup_policy_id: backup policy id
        :param resources: resources to bound, should be a list of volume id
        :return:
        """
        return self._process(session, backup_policy_id, True, resources)

    def unlink(self, session, backup_policy_id, resources):
        """unlink resources from backup policy

        :param session: openstack session
        :param backup_policy_id: backup policy id
        :param resources: resources to unbound, should be a list of volume id
        :return:
        """
        return self._process(session, backup_policy_id, False, resources)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import exceptions
from openstack import resource


class RecyclePolicy(resource.Resource):
    base_path = 'instances/recycle-policy'
    resource_key = 'recycle_policy'

    requires_id = False

    allow_fetch = True
    allow_create = True
    create_method = "PUT"

    enabled = resource.Body('enabled', type=bool)
    retention_period_in_days = resource.Body('retention_period_in_days')

    def fetch(self, session, requires_id=False, base_path=None,
              error_message=None, skip_cache=False, *,
              resource_response_key=None,
              microversion=None, **params,):
        request = self._prepare_request(
            requires_id=requires_id,
            base_path=self.base_path,
        )
        session = self._get_session(session)
        if microversion is None:
            microversion = self._get_microversion(session)
        response = session.get(
            request.url,
            headers={"Content-Type": "application/json"},
            microversion=microversion,
            params=params,
            skip_cache=skip_cache,
        )
        exceptions.raise_from_response(response)
        self._translate_response(response)
        return self

    def create(self, session, prepend_key=True, base_path=None, *,
               resource_request_key=None, resource_response_key=None,
               microversion=None, **params,):
        if not self.allow_create:
            raise exceptions.MethodNotSupported(self, 'create')
        session = self._get_session(session)
        if microversion is None:
            microversion = self._get_microversion(session)
        request_kwargs = {
            "requires_id": self.requires_id,
            "prepend_key": prepend_key,
            "base_path": self.base_path,
        }
        request = self._prepare_request(**request_kwargs)
        response = session.put(
            request.url,
            json=request.body,
            headers=request.headers,
            microversion=microversion,
            params=params,
        )
        exceptions.raise_from_response(response)
        self._translate_response(response)
        return self



# Copyright 2012 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import json
import re
import sys

from openstack import exceptions


class BaseException(Exception):
    """An error occurred."""
    def __init__(self, message=None):
        self.message = message

    def __str__(self):
        return self.message or self.__class__.__doc__


class CommandError(BaseException):
    """Invalid usage of CLI."""


class InvalidEndpoint(BaseException):
    """The provided endpoint is invalid."""


class CommunicationError(BaseException):
    """Unable to communicate with server."""


class ClientException(Exception):
    """DEPRECATED!"""


class HTTPException(ClientException):
    """Base exception for all HTTP-derived exceptions."""
    code = 'N/A'

    def __init__(self, details=None):
        self.details = details or self.__class__.__name__

    def __str__(self):
        return "%s (HTTP %s)" % (self.details, self.code)


class HTTPMultipleChoices(HTTPException):
    code = 300

    def __str__(self):
        self.details = ("Requested version of OpenStack Images API is not "
                        "available.")
        return "%s (HTTP %s) %s" % (self.__class__.__name__, self.code,
                                    self.details)


class BadRequest(HTTPException):
    """DEPRECATED!"""
    code = 400


class HTTPBadRequest(BadRequest):
    pass


class Unauthorized(HTTPException):
    """DEPRECATED!"""
    code = 401


class HTTPUnauthorized(Unauthorized):
    pass


class Forbidden(HTTPException):
    """DEPRECATED!"""
    code = 403


class HTTPForbidden(Forbidden):
    pass


class NotFound(HTTPException):
    """DEPRECATED!"""
    code = 404


class HTTPNotFound(NotFound):
    pass


class HTTPMethodNotAllowed(HTTPException):
    code = 405


class Conflict(HTTPException):
    """DEPRECATED!"""
    code = 409


class HTTPConflict(Conflict):
    pass


class OverLimit(HTTPException):
    """DEPRECATED!"""
    code = 413


class HTTPOverLimit(OverLimit):
    pass


class HTTPInternalServerError(HTTPException):
    code = 500


class HTTPNotImplemented(HTTPException):
    code = 501


class HTTPBadGateway(HTTPException):
    code = 502


class ServiceUnavailable(HTTPException):
    """DEPRECATED!"""
    code = 503


class HTTPServiceUnavailable(ServiceUnavailable):
    pass


# NOTE(bcwaldon): Build a mapping of HTTP codes to corresponding exception
# classes
_code_map = {}
for obj_name in dir(sys.modules[__name__]):
    if obj_name.startswith('HTTP'):
        obj = getattr(sys.modules[__name__], obj_name)
        _code_map[obj.code] = obj


def from_response(response, body=None):
    """Return an instance of an HTTPException based on httplib response."""
    cls = _code_map.get(response.status_code, HTTPException)
    if body and 'json' in response.headers['content-type']:
        # Iterate over the nested objects and retrieve the "message" attribute.
        messages = [obj.get('message') for obj in response.json().values()]
        # Join all of the messages together nicely and filter out any objects
        # that don't have a "message" attr.
        details = '\n'.join(i for i in messages if i is not None)
        return cls(details=details)
    elif body and 'html' in response.headers['content-type']:
        # Split the lines, strip whitespace and inline HTML from the response.
        details = [re.sub(r'<.+?>', '', i.strip())
                   for i in response.text.splitlines()]
        details = [i for i in details if i]
        # Remove duplicates from the list.
        details_seen = set()
        details_temp = []
        for i in details:
            if i not in details_seen:
                details_temp.append(i)
                details_seen.add(i)
        # Return joined string separated by colons.
        details = ': '.join(details_temp)
        return cls(details=details)
    elif body:
        body = body.decode('utf-8')
        details = body.replace('\n\n', '\n')
        return cls(details=details)

    return cls()


class NoTokenLookupException(Exception):
    """DEPRECATED!"""
    pass


class EndpointNotFound(Exception):
    """DEPRECATED!"""
    pass


class SSLConfigurationError(BaseException):
    pass


class SSLCertificateError(BaseException):
    pass


def _extract_message(obj):
    if isinstance(obj, dict):
        # Most of services: compute, network
        if obj.get('message'):
            return obj['message']
        # Ironic starting with Stein
        elif obj.get('faultstring'):
            return obj['faultstring']
        elif obj.get('error_code'):
            return obj['error_code']
        elif obj.get('error_msg'):
            return obj['error_msg']
        # Stupid VBS service returns another incompatibility
        elif obj.get('badRequest'):
            return obj['badRequest']
        else:
            # We are in the dead end. Not to hide potential error info take it
            # as it is
            return obj

    elif isinstance(obj, str):
        # Ironic before Stein has double JSON encoding, nobody remembers why.
        try:
            obj = json.loads(obj)
        except Exception:
            pass
        else:
            return _extract_message(obj)


def raise_from_response(response, error_message=None):
    """Raise an instance of an HTTPException based on keystoneauth response."""
    if response.status_code < 400:
        return

    if response.status_code == 409:
        cls = exceptions.ConflictException
    elif response.status_code == 404:
        cls = exceptions.NotFoundException
    elif response.status_code == 400:
        cls = exceptions.BadRequestException
    else:
        cls = exceptions.HttpException

    details = None
    content_type = response.headers.get('content-type', '')
    if response.content and 'application/json' in content_type:
        # Iterate over the nested objects to retrieve "message" attribute.

        try:
            content = response.json()
            messages = [_extract_message(obj) for obj in content.values()]
            if not any(messages):
                # Exception dict may be the root dict in projects that use WSME
                messages = [_extract_message(content)]

            # Join all of the messages together nicely and filter out any
            # objects that don't have a "message" attr.
            details = '\n'.join(msg for msg in messages if msg)
        except Exception:
            details = response.text
    elif response.content and 'text/html' in content_type:
        # Split the lines, strip whitespace and inline HTML from the response.
        details = [re.sub(r'<.+?>', '', i.strip())
                   for i in response.text.splitlines()]
        details = list(set([msg for msg in details if msg]))
        # Return joined string separated by colons.
        details = ': '.join(details)
    if not details:
        details = response.reason if response.reason else response.text

    http_status = response.status_code
    request_id = response.headers.get('x-openstack-request-id')

    # sdk.exception define default for message to Error, but we need
    # have a better info
    if not error_message and details:
        error_message = details

    raise cls(
        message=error_message, response=response, details=details,
        http_status=http_status, request_id=request_id
    )



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""
Create Auto-Scaling Policy.
"""
import openstack

openstack.enable_logging(True)
conn = openstack.connect(cloud='otc')

attrs = {
    "scaling_group_id": "as_group_id",
    "scaling_policy_name": "as_policy_name",
    "scaling_policy_type": "ALARM",
    "alarm_id": "cloudeye_alarm_id",
    "scheduled_policy": {},
    "cool_down_time": 300,
    "scaling_policy_action": {
        "operation": "ADD",
        "instance_number": 1
    }
}

policy = conn.auto_scaling.create_policy(**attrs)
print(policy)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource
from openstack import utils
from openstack import exceptions


class Attachment(resource.Resource):
    #: Properties
    #: Replication pair ID
    replication = resource.Body('replication')
    #: Protected instance attached disk name
    device = resource.Body('device')


class Metadata(resource.Resource):
    #: Properties
    #: Resource status (frozen or not)
    system_frozen = resource.Body('__system__frozen',
                                  type=bool)


class TagSpec(resource.Resource):
    #: Properties
    #: Tag key
    key = resource.Body('key')
    #: Tag value
    value = resource.Body('value')


class ProtectedInstance(resource.Resource):
    """SDRS Protected Instance Resource"""
    resource_key = 'protected_instance'
    resources_key = 'protected_instances'
    base_path = '/protected-instances'

    # capabilities
    allow_create = True
    allow_list = True
    allow_fetch = True
    allow_delete = True
    allow_commit = True

    _query_mapping = resource.QueryParameters(
        'availability_zone', 'limit', 'marker', 'name', 'offset',
        'protected_instance_ids', 'query_type', 'server_group_id',
        'server_group_ids', 'status')

    #: Properties
    #: Attached replication pair info
    attachment = resource.Body('attachment', type=list, list_type=Attachment)
    #: Creation time
    created_at = resource.Body('created_at')
    #: Protected instance description
    description = resource.Body('description')
    #: Specifies whether to delete EIP on DR site server
    #: Default: False
    delete_target_eip = resource.Body('delete_target_eip', type=bool)
    #: Specifies whether to delete server on DR site server
    #: Default: False
    delete_target_server = resource.Body('delete_target_server', type=bool)
    #: DR site server flavor ID
    flavorRef = resource.Body('flavorRef')
    #: Protected instance ID
    id = resource.Body('id')
    #: Created task job ID
    job_id = resource.Body('job_id')
    #: Protected instance metadata
    metadata = resource.Body('metadata', type=Metadata)
    #: Protected instance name
    name = resource.Body('name')
    #: Network ID of the subnet for primary NIC on
    #: DR site server
    primary_subnet_id = resource.Body('primary_subnet_id')
    #: IP address of the primary NIC on DR site server
    primary_ip_address = resource.Body('primary_ip_address')
    #: Production site of protection group
    priority_station = resource.Body('priority_station')
    #: Protected instance synch progress
    progress = resource.Body('progress', type=int)
    #: Protection group ID
    server_group_id = resource.Body('server_group_id')
    #: Production site ECS ID
    server_id = resource.Body('server_id')
    #: Production site server ID
    source_server = resource.Body('source_server')
    #: Protected instance status
    status = resource.Body('status')
    #: Instance tag list
    tags = resource.Body('tags', type=list, list_type=TagSpec)
    #: DR site server ID
    target_server = resource.Body('target_server')
    #: Update time
    updated_at = resource.Body('updated_at')

    def delete(self, session,
               delete_target_server=False, delete_target_eip=False,
               ignore_missing=True):
        """Delete the remote resource based on this instance.

        This function overrides default Resource.delete to enable params

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param bool delete_target_server: Specifies whether target
            ECS should be deleted after protection group deletion
        :param bool delete_target_eip: Specifies whether target
            ECS should be deleted after protection group deletion
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the group does not exist.
            When set to ``True``, no exception will be set when attempting
            to delete a nonexistent protected instance
        :return: This :class:`Group` instance.
        """
        body = {
            'delete_target_server': delete_target_server,
            'delete_target_eip': delete_target_eip
        }
        request = self._prepare_request()
        response = session.delete(request.url,
                                  json=body)
        try:
            self._translate_response(response, has_body=True)
        except exceptions.ResourceNotFound:
            if ignore_missing:
                return None
            raise
        return self

    def attach_pair(self, session, protected_instance,
                    replication_id, device='/dev/vdb'):
        """Method to attach replication pair to the specified
            protected instance

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param str protected_instance: ID of protected instance
            on which replication pair will be attached
        :param str replication_id: ID of replication pair to be
            attached
        :param str device: disk device name of a replication pair
        """
        url = utils.urljoin(self.base_path,
                            protected_instance,
                            'attachreplication')
        body = {
            "replicationAttachment": {
                "replication_id": replication_id,
                "device": device
            }
        }

        return session.post(url, json=body)

    def detach_pair(self, session, protected_instance,
                    replication_id):
        """Method to detach replication pair from the specified
            protected instance

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param str protected_instance: ID of protected instance
            from which replication pair will be detached
        :param str replication_id: ID of replication pair to be
            detached
        """
        url = utils.urljoin(self.base_path,
                            protected_instance,
                            'detachreplication',
                            replication_id)

        return session.delete(url)

    def add_nic(self, session, protected_instance, subnet_id,
                security_groups=None, ip_address=None):
        """Method to add NIC to protected instance

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param str protected_instance: ID of protected instance
            for which NIC will be added
        :param str subnet_id: Subnet ID of the NIC to be added
        :param list security_groups: list of security groups
            to be added for NIC in format 'id': 'value'
        :param str ip_address: IP address of NIC
        """
        body = {
            'subnet_id': subnet_id
        }
        if security_groups:
            body['security_groups'] = security_groups
        if ip_address:
            body['ip_address'] = ip_address

        url = utils.urljoin(self.base_path,
                            protected_instance,
                            'nic')
        return session.post(url, json=body)

    def delete_nic(self, session, protected_instance,
                   nic_id):
        """Method to remove NIC to protected instance

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param str protected_instance: ID of protected instance
            for which NIC will be added
        :param str nic_id: ID of Network interface card
        """
        body = {
            'nic_id': nic_id
        }
        url = utils.urljoin(self.base_path,
                            protected_instance,
                            'nic/delete')
        return session.post(url, json=body)

    def modify_instance(self, session,
                        protected_instance, flavor=None,
                        production_flavor=None, dr_flavor=None):
        """Method to modify server specifications

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param str protected_instance: ID of protected instance
        :param str flavor: flavor ID for both production and DR sites
        :param str production_flavor: flavor ID for production site
            If 'flavor' is specified this parameter doesn't take effect
        :param str dr_flavor: flavor ID for DR site
            If 'flavor' is specified this parameter doesn't take effect
        """
        body = {
            'resize': {}
        }
        if flavor:
            body['resize']['flavorRef'] = flavor
        if production_flavor:
            body['resize']['production_flavorRef'] = production_flavor
        if dr_flavor:
            body['resize']['dr_flavorRef'] = dr_flavor

        url = utils.urljoin(self.base_path,
                            protected_instance,
                            'resize')
        return session.post(url, json=body)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from otcextensions.tests.functional import base
import uuid


class TestApiG(base.BaseFunctionalTest):
    gateway = None

    def setUp(self):
        super(TestApiG, self).setUp()
        self.client = self.conn.apig

    def get_attrs(self):
        all_vpc = list(self.conn.vpc.vpcs())
        vpc = all_vpc[0]
        all_subnets = list(self.conn.vpc.subnets(vpc_id=vpc.id))
        subnet = all_subnets[0]
        security_groups = list(self.conn.network.security_groups())
        security_group = security_groups[0]
        available_zone_ids = ["eu-de-01"]
        nmb = uuid.uuid4().hex[:8]
        return {
            'instance_name': 'test_gateway_{}'.format(nmb),
            'spec_id': 'BASIC',
            'vpc_id': vpc.id,
            'subnet_id': subnet.id,
            'security_group_id': security_group.id,
            'available_zone_ids': available_zone_ids,
        }

    def create_gateway(self):
        if not TestApiG.gateway:
            attrs = self.get_attrs()
            TestApiG.gateway = self.client.create_gateway(**attrs)
            TestApiG.gateway = self.client.wait_for_gateway(
                TestApiG.gateway.instance_id)

    def delete_gateway(self):
        if TestApiG.gateway:
            self.client.delete_gateway(TestApiG.gateway)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import uuid
from otcextensions.sdk.function_graph.v2 import function
from otcextensions.tests.functional import base

from openstack import _log

_logger = _log.setup_logging('openstack')


class TestFunctionEvent(base.BaseFunctionalTest):
    ID = None
    uuid = uuid.uuid4().hex[:8]

    def setUp(self):
        super(TestFunctionEvent, self).setUp()
        self.attrs = {
            'func_name': 'test-function-' + self.uuid,
            'package': 'default',
            'runtime': 'Python3.9',
            'handler': 'index.handler',
            'timeout': 30,
            'memory_size': 128,
            'code_type': 'inline',
            'func_code': {
                'file': 'CmltcG9ydCBqc29uCgpkZWYgaGFuZGxlcihldmVudCwgY29udGV4d'
                        'Ck6CiAgICB0cnk6CiAgICAgICAgIyBQYXJzZSB0aGUgaW5wdXQgZX'
                        'ZlbnQgKGRhdGEpCiAgICAgICAgaW5wdXRfZGF0YSA9IGpzb24ubG9'
                        'hZHMoZXZlbnQpCiAgICAgICAgCiAgICAgICAgIyBQcmludCB0aGUg'
                        'cHJvdmlkZWQgZGF0YQogICAgICAgIHByaW50KCJSZWNlaXZlZCBkY'
                        'XRhOiIsIGlucHV0X2RhdGEpCiAgICAgICAgCiAgICAgICAgIyBSZX'
                        'R1cm4gYSByZXNwb25zZQogICAgICAgIHJlc3BvbnNlID0gewogICA'
                        'gICAgICAgICAic3RhdHVzQ29kZSI6IDIwMCwKICAgICAgICAgICAg'
                        'ImJvZHkiOiBmIkRhdGEgcmVjZWl2ZWQ6IHtpbnB1dF9kYXRhfSIKI'
                        'CAgICAgICB9CiAgICAgICAgcmV0dXJuIGpzb24uZHVtcHMocmVzcG'
                        '9uc2UpCgogICAgZXhjZXB0IEV4Y2VwdGlvbiBhcyBlOgogICAgICA'
                        'gICMgSGFuZGxlIGVycm9ycwogICAgICAgIGVycm9yX3Jlc3BvbnNl'
                        'ID0gewogICAgICAgICAgICAic3RhdHVzQ29kZSI6IDUwMCwKICAgI'
                        'CAgICAgICAgImJvZHkiOiBmIkVycm9yOiB7c3RyKGUpfSIKICAgIC'
                        'AgICB9CiAgICAgICAgcmV0dXJuIGpzb24uZHVtcHMoZXJyb3JfcmV'
                        'zcG9uc2UpCg=='
            }
        }

        self.NAME = 'test-function-' + self.uuid
        self.UPDATE_NAME = 'test-function-upd-' + self.uuid
        self.function = self.conn.functiongraph.create_function(**self.attrs)
        assert isinstance(self.function, function.Function)
        self.assertEqual(self.NAME, self.function.func_name)

        self.event_attrs = {
            'name': 'event-xx',
            'content': 'eyJrIjoidiJ9'
        }
        self.event = self.conn.functiongraph.create_event(
            self.function, **self.event_attrs
        )
        self.ID = self.event.id

        self.addCleanup(
            self.conn.functiongraph.delete_function,
            self.function
        )
        self.addCleanup(
            self.conn.functiongraph.delete_event,
            self.function, self.event
        )

    def test_function_events(self):
        elist = list(self.conn.functiongraph.events(
            func_urn=self.function.func_urn))
        self.assertIn(self.ID, elist[0].id)

    def test_get_function_event(self):
        e = self.conn.functiongraph.get_event(
            self.function, self.event)
        self.assertIn(self.ID, e.id)

    def test_update_function_event(self):
        event_attrs = {
            'content': 'ewogICAgImJvZHkiOiAiIiwKICAgICJy'
        }
        updated = self.conn.functiongraph.update_event(
            self.function, self.event, **event_attrs)
        self.assertIn(event_attrs['content'], updated.content)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
'''
Create CBR Checkpoint
'''
import openstack
from otcextensions import sdk


openstack.enable_logging(True)
conn = openstack.connect(cloud='otc')
sdk.register_otc_extensions(conn)


attrs = {
    'parameters': {
        'auto_trigger': False,
        'description': 'backup_description',
        'incremental': False,
        'name': 'manual_backup',
        'resources': [
            'ecs_id'
        ]
    },
    'vault_id': 'vault_id'
}
checkpoint = conn.cbr.create_checkpoint(**attrs)
conn.cbr.wait_for_checkpoint(checkpoint)
print(checkpoint)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from unittest import mock

from otcextensions.sdk.dms.v1 import _proxy
from otcextensions.sdk.dms.v1 import az as _az
from otcextensions.sdk.dms.v1 import group as _group
from otcextensions.sdk.dms.v1 import instance as _instance
from otcextensions.sdk.dms.v1 import maintenance_window as _mw
from otcextensions.sdk.dms.v1 import message as _message
from otcextensions.sdk.dms.v1 import product as _product
from otcextensions.sdk.dms.v1 import queue as _queue
from otcextensions.sdk.dms.v1 import topic as _topic

from openstack.tests.unit import test_proxy_base

GROUP_ID = 'g-5ec247fd-d4a2-4d4f-9876-e4ff3280c461'


class TestDMSProxy(test_proxy_base.TestProxyBase):
    def setUp(self):
        super(TestDMSProxy, self).setUp()
        self.proxy = _proxy.Proxy(self.session)

    def test_create_queue(self):
        self.verify_create(
            self.proxy.create_queue,
            _queue.Queue,
        )

    def test_queues(self):
        self.verify_list(
            self.proxy.queues,
            _queue.Queue,
            expected_kwargs={
                'paginated': False
            }
        )

    def test_queues_qp(self):
        self.verify_list(
            self.proxy.queues,
            _queue.Queue,
            method_kwargs={
                'include_deadletter': True,
            },
            expected_kwargs={
                'include_deadletter': True,
                'paginated': False
            }
        )

    def test_get_queue(self):
        self.verify_get(
            self.proxy.get_queue,
            _queue.Queue
        )

    def test_delete_queue(self):
        self.verify_delete(
            self.proxy.delete_queue,
            _queue.Queue,
            False
        )

    def test_delete_queue_ignore(self):
        self.verify_delete(
            self.proxy.delete_queue,
            _queue.Queue,
            True
        )

    def test_create_group(self):
        self.verify_create(
            self.proxy.create_group,
            _group.Group,
            method_kwargs={'queue': 'qip', 'name': 'grp'},
            expected_kwargs={'queue_id': 'qip', 'name': 'grp'})

    def test_groups(self):
        self.verify_list(
            self.proxy.groups,
            _group.Group,
            method_kwargs={'queue': 'qid'},
            expected_kwargs={
                'queue_id': 'qid',
                'paginated': False}
        )

    def test_delete_group(self):
        self.verify_delete(
            self.proxy.delete_group,
            _group.Group,
            False,
            method_args=['QID', 'resource_or_id'],
            expected_kwargs={
                'ignore_missing': False,
                'queue_id': 'QID'
            },
            expected_args=['resource_or_id']
        )

    def test_delete_group_ignore(self):
        self.verify_delete(
            self.proxy.delete_group,
            _group.Group,
            ignore_missing=True,
            method_args=['QID', 'resource_or_id'],
            expected_kwargs={
                'ignore_missing': True,
                'queue_id': 'QID'
            },
            expected_args=['resource_or_id']
        )

    ######
    # Messages

    def test_send_messages_dict(self):
        self.verify_create(
            self.proxy.send_messages,
            _message.Messages,
            method_kwargs={
                'queue': 'qid',
                'messages': [{
                    'body': 'b1',
                    'attributes': {
                        'attribute1': 'value1',
                        'attribute2': 'value2'}
                }]
            },
            expected_kwargs={
                'queue_id': 'qid',
                'messages': [{
                    'body': 'b1',
                    'attributes': {
                        'attribute1': 'value1',
                        'attribute2': 'value2'}
                }],
                'return_id': False
            }
        )

    def test_send_messages_msg(self):
        self.verify_create(
            self.proxy.send_messages,
            _message.Messages,
            method_kwargs={
                'queue': 'qid',
                'messages': [
                    _message.Message(
                        body='b1',
                        attributes={
                            'attribute1': 'value1',
                            'attribute2': 'value2'}
                    )
                ]
            },
            expected_kwargs={
                'queue_id': 'qid',
                'messages': [{
                    'body': 'b1',
                    'attributes': {
                        'attribute1': 'value1',
                        'attribute2': 'value2'}
                }],
                'return_id': False
            }
        )

    def test_send_message(self):
        value = _message.Message(id='1')
        self._verify(
            'openstack.proxy.Proxy._create',
            self.proxy.send_message,
            method_kwargs={
                'queue': 'qid',
                'body': 'b1',
                'attributes': {
                    'p1': 'v1'
                }
            },
            expected_kwargs={
                'queue_id': 'qid',
                'messages': [
                    {'attributes': {'p1': 'v1'}, 'body': 'b1'}
                ],
                'return_id': True
            },
            method_result=value,
            expected_result=_message.Messages(
                messages=[value]),
            expected_args=[_message.Messages]
        )

    def test_consume_message(self):
        self.verify_list(
            self.proxy.consume_message,
            _message.Message,
            method_kwargs={
                'queue': 'qid',
                'group': 'gid'
            },
            expected_kwargs={
                'queue_id': 'qid',
                'group_id': 'gid'
            },
            base_path='/queues/%(queue_id)s/groups/%(group_id)s/messages'
        )

    def test_ack_consumed_message(self):
        pass

    def test_quotas(self):
        pass

    ######
    # Instances
    def test_instances(self):
        self.verify_list(
            self.proxy.instances,
            _instance.Instance,
            expected_kwargs={
                'paginated': False
            }
        )

    def test_create_instance(self):
        self.verify_create(
            self.proxy.create_instance,
            _instance.Instance
        )

    def test_delete_instance(self):
        self.verify_delete(
            self.proxy.delete_instance,
            _instance.Instance,
            False,
        )

    def test_delete_instance_ignore(self):
        self.verify_delete(
            self.proxy.delete_instance,
            _instance.Instance,
            True,
        )

    def test_find_instance(self):
        self.verify_find(
            self.proxy.find_instance,
            _instance.Instance
        )

    def test_get_instance(self):
        self.verify_get(
            self.proxy.get_instance,
            _instance.Instance
        )

    def test_update_instance(self):
        self.verify_update(
            self.proxy.update_instance,
            _instance.Instance
        )

    def test_restart_instance(self):
        self._verify(
            'otcextensions.sdk.dms.v1.instance.Instance._action',
            self.proxy.restart_instance,
            method_args=['value'],
            expected_args=[self.proxy, 'restart', ['value']]
        )

    def test_restart_instances(self):
        self._verify(
            'otcextensions.sdk.dms.v1.instance.Instance._action',
            self.proxy.restart_instances,
            method_args=[['1', '2']],
            expected_args=[self.proxy, 'restart', ['1', '2']]
        )

    def test_delete_failed(self):
        self._verify(
            'otcextensions.sdk.dms.v1.instance.Instance.delete_failed',
            self.proxy.delete_failed,
            method_args=[]
        )

    def test_delete_batch(self):
        self._verify(
            'otcextensions.sdk.dms.v1.instance.Instance._action',
            self.proxy.delete_batch,
            method_args=[['1', '2']],
            expected_args=[self.proxy, 'delete', ['1', '2']]
        )

    def test_create_topic(self):
        self.verify_create(
            self.proxy.create_topic,
            _topic.Topic,
            method_args=['iid'],
            expected_kwargs={'instance_id': 'iid', 'x': 1, 'y': 2, 'z': 3},
            expected_args=[]
        )

    @mock.patch('otcextensions.sdk.dms.v1._proxy.Proxy.post')
    def test_delete_topics(self, post_mock):
        response = mock.Mock()
        response.status_code = 200
        post_mock.return_value = response
        self.proxy.delete_topic('instance', ['t1', 't2'])

        post_mock.assert_called_with(
            '/instances/instance/topics/delete',
            json={'topics': ['t1', 't2']})

        self.proxy.delete_topic('instance', 't1')

        post_mock.assert_called_with(
            '/instances/instance/topics/delete',
            json={'topics': ['t1']})

    def test_topics(self):
        self.verify_list(
            self.proxy.topics,
            _topic.Topic,
            method_args=['iid'],
            expected_kwargs={
                'instance_id': 'iid'
            },
            expected_args=[]
        )

    # Misc
    def test_az(self):
        self.verify_list(
            self.proxy.availability_zones,
            _az.AvailabilityZone,
        )

    def test_products(self):
        self.verify_list(
            self.proxy.products,
            _product.Product
        )

    def test_mws(self):
        self.verify_list(
            self.proxy.maintenance_windows,
            _mw.MaintenanceWindow
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.unit import base

from otcextensions.sdk.vlb.v3 import flavor

EXAMPLE = {
    'id': 'id',
    'info': {},
    'flavor_sold_out': True,
    'name': 'name',
    'shared': True,
    'project_id': 'project-id',
    'type': 'type-id',
}


class TestLoadBalancer(base.TestCase):

    def test_basic(self):
        sot = flavor.Flavor()
        path = '/elb/flavors'
        self.assertEqual(path, sot.base_path)
        self.assertFalse(sot.allow_create)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_fetch)
        self.assertFalse(sot.allow_commit)
        self.assertFalse(sot.allow_delete)

    def test_make_it(self):
        sot = flavor.Flavor(**EXAMPLE)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['info'], sot.info)
        self.assertEqual(EXAMPLE['flavor_sold_out'], sot.flavor_sold_out)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['shared'], sot.shared)
        self.assertEqual(EXAMPLE['project_id'], sot.project_id)
        self.assertEqual(EXAMPLE['type'], sot.type)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import uuid

from otcextensions.tests.functional.sdk.apig import TestApiG


class TestThrottleBind(TestApiG):
    def setUp(self):
        super(TestThrottleBind, self).setUp()
        self.suffix = uuid.uuid4().hex[:4]

        self.create_gateway()
        self.gateway_id = TestThrottleBind.gateway.id
        # self.gateway_id = "be76ca6de5fe4aa7af503c03b3b44dea"

        self.attrs = {
            "api_call_limits": 100,
            "app_call_limits": 60,
            "enable_adaptive_control": "FALSE",
            "ip_call_limits": 60,
            "name": f"throttle_{self.suffix}",
            "time_interval": 1,
            "time_unit": "SECOND",
            "type": 1,
            "user_call_limits": 60
        }
        self.policy = self.client.create_throttling_policy(
            gateway=self.gateway_id,
            **self.attrs
        )

        group_attrs = {
            "name": f"api_group_{self.suffix}",
            "remark": "API group 1"
        }
        self.group = self.client.create_api_group(
            gateway=self.gateway_id,
            **group_attrs
        )
        self.assertIsNotNone(self.group.id)

        api_attrs = {
            "group_id": self.group.id,
            "name": f"test_api_{self.suffix}",
            "auth_type": "IAM",
            "backend_type": "HTTP",
            "req_protocol": "HTTP",
            "req_uri": "/test/http",
            "remark": "Mock backend API",
            "type": 2,
            "req_method": "GET",
            "result_normal_sample": "Example success response",
            "result_failure_sample": "Example failure response",
            "tags": ["httpApi"],
            "backend_api": {
                "req_protocol": "HTTP",
                "req_method": "GET",
                "req_uri": "/test/benchmark",
                "timeout": 5000,
                "retry_count": "-1",
                "url_domain": "192.168.189.156:12346"
            },
        }
        self.api = self.client.create_api(
            gateway=self.gateway_id,
            **api_attrs
        )
        self.environment = self.client.create_environment(
            gateway=self.gateway_id,
            name=f"testPub{self.suffix}",
            remark="test publish"
        )
        self.publish = self.client.publish_api(
            gateway=self.gateway_id,
            api=self.api.id,
            env=self.environment,
            remark="publish"
        )

        self.attrs = {
            "throttle_id": self.policy.id,
            "publish_ids": [self.publish.publish_id]
        }
        self.bind = self.client.bind_throttling_policy(
            gateway=self.gateway_id,
            **self.attrs
        )

        self.addCleanup(
            self.client.delete_api_group,
            gateway=self.gateway_id,
            api_group=self.group.id,
        )
        self.addCleanup(
            self.client.delete_environment,
            gateway=self.gateway_id,
            environment=self.environment,
        )
        self.addCleanup(
            self.client.delete_throttling_policy,
            gateway=self.gateway_id,
            policy=self.policy
        )
        self.addCleanup(
            self.client.delete_api,
            gateway=self.gateway_id,
            api=self.api
        )
        self.addCleanup(
            self.client.offline_api,
            gateway=self.gateway_id,
            api=self.api.id,
            env=self.environment,
            remark="offline"
        )
        self.addCleanup(
            self.client.unbind_throttling_policies,
            gateway=self.gateway_id,
            throttle_bindings=[self.bind.policies[0].id]
        )

        self.addCleanup(self.delete_gateway())

    def test_list_bound_throttling_policies(self):
        sign = list(self.client.bound_throttling_policies(
            gateway=self.gateway_id,
            api_id=self.api.id
        ))
        self.assertEqual(len(sign), 1)

    def test_list_bound_throttling_policy_apis(self):
        sign = list(self.client.bound_throttling_policy_apis(
            gateway=self.gateway_id,
            throttle_id=self.policy.id
        ))
        self.assertEqual(len(sign), 1)

    def test_list_not_bound_throttling_policy_apis(self):
        sign = list(self.client.not_bound_throttling_policy_apis(
            gateway=self.gateway_id,
            throttle_id=self.policy.id
        ))
        self.assertEqual(len(sign), 0)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from keystoneauth1 import adapter

import mock

from openstack import exceptions
from openstack.tests.unit import base

from otcextensions.sdk.auto_scaling.v1 import instance

EXAMPLE_LIST = {
    'limit': 10,
    'total_number': 1,
    'start_number': 0,
    'scaling_group_instances': [
        {
            'instance_id': 'b25c1589-c96c-465b-9fef-d06540d1945c',
            'scaling_group_id': 'e5d27f5c-dd76-4a61-b4bc-a67c5686719a',
            'scaling_group_name': 'discuz',
            'life_cycle_state': 'INSERVICE',
            'health_status': 'NORMAL',
            'scaling_configuration_name': 'discuz',
            'scaling_configuration_id': 'ca3dcd84-d197-4c4f-af2a-cf8ba39696ac',
            'create_time': '2015-07-23T06:47:33Z',
            'instance_name': 'discuz_3D210808'
        }
    ]
}


class TestInstance(base.TestCase):

    def setUp(self):
        super(TestInstance, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.get = mock.Mock()
        self.sess.post = mock.Mock()
        self.sess.delete = mock.Mock()
        self.sess.put = mock.Mock()
        self.sot = instance.Instance()

    def test_basic(self):
        sot = instance.Instance()
        self.assertEqual('scaling_group_instance', sot.resource_key)
        self.assertEqual('scaling_group_instances', sot.resources_key)
        self.assertEqual('/scaling_group_instance', sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertFalse(sot.allow_create)
        self.assertFalse(sot.allow_fetch)
        self.assertFalse(sot.allow_commit)
        self.assertTrue(sot.allow_delete)

    def test_make_it(self):
        obj = EXAMPLE_LIST['scaling_group_instances'][0].copy()
        sot = instance.Instance.existing(**obj)
        self.assertEqual(obj['instance_id'], sot.id)
        self.assertEqual(obj['instance_name'], sot.name)
        self.assertEqual(obj['scaling_group_name'], sot.scaling_group_name)
        self.assertEqual(obj['life_cycle_state'], sot.lifecycle_state)
        self.assertEqual(obj['health_status'], sot.health_status)
        self.assertEqual(obj['scaling_configuration_name'],
                         sot.scaling_configuration_name)
        self.assertEqual(obj['scaling_configuration_id'],
                         sot.scaling_configuration_id)
        self.assertEqual(obj['create_time'], sot.create_time)

    def test_batch_action_act_check(self):
        mock_response = mock.Mock()
        mock_response.status_code = 204
        mock_response.json.return_value = None

        self.sess.post.return_value = mock_response

        sot = instance.Instance.existing(scaling_group_id='grp_id')

        obj_list = ['i1', 'i2']

        self.assertRaises(
            exceptions.SDKException, sot.batch_action,
            session=self.sess, instances=obj_list, action='f')

    def test_batch_action_add(self):
        mock_response = mock.Mock()
        mock_response.status_code = 204
        mock_response.json.return_value = None

        self.sess.post.return_value = mock_response

        sot = instance.Instance.existing(scaling_group_id='grp_id')

        obj_list = ['i1', 'i2']

        sot.batch_action(self.sess, obj_list, 'ADD')

        self.sess.post.assert_called_once_with(
            'scaling_group_instance/grp_id/action',
            json={
                'action': 'ADD',
                'instances_id': obj_list,
            }
        )

    def test_batch_action_remove(self):
        mock_response = mock.Mock()
        mock_response.status_code = 204
        mock_response.json.return_value = None

        self.sess.post.return_value = mock_response

        sot = instance.Instance.existing(scaling_group_id='grp_id')

        obj_list = ['i1', 'i2']

        sot.batch_action(self.sess, obj_list, 'REMOVE', delete_instance=True)

        self.sess.post.assert_called_once_with(
            'scaling_group_instance/grp_id/action',
            json={
                'action': 'REMOVE',
                'instances_id': obj_list,
                'instance_delete': 'yes'
            }
        )

    def test_batch_action_protect(self):
        mock_response = mock.Mock()
        mock_response.status_code = 204
        mock_response.json.return_value = None

        self.sess.post.return_value = mock_response

        sot = instance.Instance.existing(scaling_group_id='grp_id')

        obj_list = ['i1', 'i2']

        sot.batch_action(self.sess, obj_list, 'PROTECT')

        self.sess.post.assert_called_once_with(
            'scaling_group_instance/grp_id/action',
            json={
                'action': 'PROTECT',
                'instances_id': obj_list,
            }
        )

    def test_batch_action_unprotect(self):
        mock_response = mock.Mock()
        mock_response.status_code = 204
        mock_response.json.return_value = None

        self.sess.post.return_value = mock_response

        sot = instance.Instance.existing(scaling_group_id='grp_id')

        obj_list = ['i1', 'i2']

        sot.batch_action(self.sess, obj_list, 'UNPROTECT')

        self.sess.post.assert_called_once_with(
            'scaling_group_instance/grp_id/action',
            json={
                'action': 'UNPROTECT',
                'instances_id': obj_list,
            }
        )

    def test_batch_action_delete_instance_not_expected(self):
        mock_response = mock.Mock()
        mock_response.status_code = 204
        mock_response.json.return_value = None

        self.sess.post.return_value = mock_response

        sot = instance.Instance.existing(scaling_group_id='grp_id')

        obj_list = ['i1', 'i2']

        self.assertRaises(
            exceptions.SDKException, sot.batch_action,
            session=self.sess, instances=obj_list,
            action='PROTECT', delete_instance=True)

    def test_remove(self):
        mock_response = mock.Mock()
        mock_response.status_code = 204
        mock_response.json.return_value = None

        self.sess.delete.return_value = mock_response

        sot = instance.Instance.existing(id='id1', scaling_group_id='grp_id')

        sot.remove(self.sess, delete_instance=True)

        self.sess.delete.assert_called_once_with(
            'scaling_group_instance/%s' % sot.id,
            params={
                'instance_delete': 'yes'
            }
        )

    def test_remove_default(self):
        mock_response = mock.Mock()
        mock_response.status_code = 204
        mock_response.json.return_value = None

        self.sess.delete.return_value = mock_response

        sot = instance.Instance.existing(id='id1', scaling_group_id='grp_id')

        sot.remove(self.sess)

        self.sess.delete.assert_called_once_with(
            'scaling_group_instance/%s' % sot.id,
            params={
                'instance_delete': 'no'
            }
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
from openstack import exceptions
from openstack import resource


class Resource(resource.Resource):

    @classmethod
    def list(
        cls,
        session,
        paginated=True,
        base_path=None,
        allow_unknown_params=False,
        *,
        microversion=None,
        **params,
    ):
        """This method is a generator which yields resource objects.

        This resource object list generator handles pagination and takes query
        params for response filtering.

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param bool paginated: ``True`` if a GET to this resource returns
            a paginated series of responses, or ``False`` if a GET returns only
            one page of data. **When paginated is False only one page of data
            will be returned regardless of the API's support of pagination.**
        :param str base_path: Base part of the URI for listing resources, if
            different from :data:`~openstack.resource.Resource.base_path`.
        :param bool allow_unknown_params: ``True`` to accept, but discard
            unknown query parameters. This allows getting list of 'filters' and
            passing everything known to the server. ``False`` will result in
            validation exception when unknown query parameters are passed.
        :param str microversion: API version to override the negotiated one.
        :param dict params: These keyword arguments are passed through the
            :meth:`~openstack.resource.QueryParamter._transpose` method
            to find if any of them match expected query parameters to be sent
            in the *params* argument to
            :meth:`~keystoneauth1.adapter.Adapter.get`. They are additionally
            checked against the :data:`~openstack.resource.Resource.base_path`
            format string to see if any path fragments need to be filled in by
            the contents of this argument.
            Parameters supported as filters by the server side are passed in
            the API call, remaining parameters are applied as filters to the
            retrieved results.

        :return: A generator of :class:`Resource` objects.
        :raises: :exc:`~openstack.exceptions.MethodNotSupported` if
            :data:`Resource.allow_list` is not set to ``True``.
        :raises: :exc:`~openstack.exceptions.InvalidResourceQuery` if query
            contains invalid params.
        """
        if not cls.allow_list:
            raise exceptions.MethodNotSupported(cls, "list")

        session = cls._get_session(session)

        if microversion is None:
            microversion = cls._get_microversion(session)

        if base_path is None:
            base_path = cls.base_path

        api_filters = cls._query_mapping._validate(
            params,
            base_path=base_path,
            allow_unknown_params=True,
        )
        client_filters = {}
        # Gather query parameters which are not supported by the server
        for k, v in params.items():
            if (
                # Known attr
                hasattr(cls, k)
                # Is real attr property
                and isinstance(getattr(cls, k), resource.Body)
                # not included in the query_params
                and k not in cls._query_mapping._mapping.keys()
            ):
                client_filters[k] = v
        query_params = cls._query_mapping._transpose(api_filters, cls)
        uri = base_path % params
        uri_params = {}

        for k, v in params.items():
            # We need to gather URI parts to set them on the resource later
            if hasattr(cls, k) and isinstance(getattr(cls, k), resource.URI):
                uri_params[k] = v

        def _dict_filter(f, d):
            """Dict param based filtering"""
            if not d:
                return False
            for key in f.keys():
                if isinstance(f[key], dict):
                    if not _dict_filter(f[key], d.get(key, None)):
                        return False
                elif d.get(key, None) != f[key]:
                    return False
            return True

        # Track the total number of resources yielded so we can paginate
        # swift objects
        total_yielded = 0
        offset = query_params.get("offset", 0)
        while uri:
            last_marker = query_params.pop("marker", None)
            # Copy query_params due to weird mock unittest interactions
            response = session.get(
                uri,
                headers={"Accept": "application/json"},
                params=query_params.copy(),
                microversion=microversion,
            )
            exceptions.raise_from_response(response)
            data = response.json()

            if cls.resources_key:
                resources = data[cls.resources_key]
            else:
                resources = data

            if not isinstance(resources, list):
                resources = [resources]

            marker = None
            for raw_resource in resources:
                # Do not allow keys called "self" through. Glance chose
                # to name a key "self", so we need to pop it out because
                # we can't send it through cls.existing and into the
                # Resource initializer. "self" is already the first
                # argument and is practically a reserved word.
                raw_resource.pop("self", None)
                # We want that URI props are available on the resource
                raw_resource.update(uri_params)

                value = cls.existing(
                    microversion=microversion,
                    connection=session._get_connection(),
                    **raw_resource,
                )
                marker = value.id
                filters_matched = True
                # Iterate over client filters and return only if matching
                for key in client_filters.keys():
                    if isinstance(client_filters[key], dict):
                        if not _dict_filter(
                            client_filters[key], value.get(key, None)
                        ):
                            filters_matched = False
                            break
                    elif value.get(key, None) != client_filters[key]:
                        filters_matched = False
                        break

                if filters_matched:
                    yield value
            total_yielded = total_yielded + len(resources)
            total_count = total_yielded
            if paginated and hasattr(cls, "resources_count_key"):
                total_count = data.get(cls.resources_count_key, total_count)

            if resources and paginated and total_count > total_yielded:
                try:
                    if marker == last_marker:
                        # If next page marker is same as what we were just
                        # asked something went terribly wrong. Some ancient
                        # services had bugs.
                        raise exceptions.SDKException(
                            "Endless pagination loop detected, aborting"
                        )
                except KeyError:
                    # do nothing, exception handling is cheaper then "if"
                    pass
                offset += 1
                query_params.update(offset=offset, marker=marker)
            else:
                return

    @classmethod
    def _get_one_match(cls, name_or_id, results):
        """Given a list of results, return the match"""
        for result in results:
            id_value = str(result.id)
            name_value = result.dataset_name

            if (id_value == name_or_id) or (name_value == name_or_id):
                return result



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
from openstack.tests.unit import base
from otcextensions.sdk.modelartsv1.v1 import training_job_version
from otcextensions.tests.unit.sdk.modelartsv1.v1 import examples
from otcextensions.tests.unit.sdk.utils import assert_attributes_equal

EXAMPLE = examples.TRAINING_JOB_VERSION


class TestTrainingJobVersion(base.TestCase):
    def setUp(self):
        super(TestTrainingJobVersion, self).setUp()

    def test_basic(self):
        sot = training_job_version.TrainingJobVersion()

        self.assertEqual("/training-jobs/%(job_id)s/versions", sot.base_path)
        self.assertEqual("versions", sot.resources_key)
        self.assertEqual(None, sot.resource_key)

        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_delete)
        self.assertFalse(sot.allow_commit)

    def test_make_it(self):
        sot = training_job_version.TrainingJobVersion(**EXAMPLE)

        for key, value in EXAMPLE.items():
            if key == "create_time":
                self.assertEqual(EXAMPLE[key], sot.created_at)
            elif key == "job_desc":
                self.assertEqual(EXAMPLE[key], sot.job_description)
            elif key == "start_time":
                self.assertEqual(EXAMPLE[key], sot.started_at)
            elif key == "system_metric_list":
                updated_sot_attrs = {
                    "cpuUsage": "cpu_usage",
                    "memUsage": "mem_usage",
                    "gpuUtil": "gpu_util",
                    "gpuMemUsage": "gpu_mem_usage",
                    "diskReadRate": "disk_read_rate",
                    "diskWriteRate": "disk_write_rate",
                    "recvBytesRate": "recv_bytes_rate",
                    "sendBytesRate": "send_bytes_rate",
                }

                for k1, v1 in updated_sot_attrs.items():
                    self.assertEqual(
                        getattr(sot.system_metric_list, v1),
                        EXAMPLE["system_metric_list"][k1],
                    )
            else:
                assert_attributes_equal(self, getattr(sot, key), value)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from keystoneauth1 import adapter

from unittest import mock

from openstack.tests.unit import base

from otcextensions.sdk.dms.v1 import az


JSON_DATA = {
    'id': '1d7b939b382c4c3bb3481a8ca10da768',
    'name': 'az10.dc1',
    'code': 'az10.dc1',
    'port': '8002',
    'resource_availability': True
}

JSON_LIST = {
    'regionId': 'fake_region',
    'available_zones': [{
        'id': '1d7b939b382c4c3bb3481a8ca10da768',
        'name': 'az10.dc1',
        'code': 'az10.dc1',
        'port': '8002',
        'resource_availability': 'true'
    }, {
        'id': '1d7b939b382c4c3bb3481a8ca10da769',
        'name': 'az10.dc2',
        'code': 'az10.dc2',
        'port': '8002',
        'resource_availability': 'true'
    }]
}


class TestAZ(base.TestCase):

    def setUp(self):
        super(TestAZ, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.get = mock.Mock()
        self.sess.endpoint_override = 'fake/%(project_id)s'
        self.sess._get_connection = mock.Mock()

    def test_basic(self):
        sot = az.AvailabilityZone()

        self.assertEqual('/availableZones', sot.base_path)
        self.assertEqual('available_zones', sot.resources_key)

        self.assertTrue(sot.allow_list)

    def test_make_it(self):

        sot = az.AvailabilityZone(**JSON_DATA)
        self.assertEqual(JSON_DATA['id'], sot.id)
        self.assertEqual(JSON_DATA['name'], sot.name)
        self.assertEqual(JSON_DATA['code'], sot.code)
        self.assertEqual(JSON_DATA['port'], sot.port)
        self.assertEqual(JSON_DATA['resource_availability'],
                         sot.has_available_resources)

    def test_list(self):
        sot = az.AvailabilityZone()

        response = mock.Mock()
        response.status_code = 200
        response.json = mock.Mock(return_value=JSON_LIST)
        self.sess.get.return_value = response

        rsp = list(sot.list(self.sess))

        self.sess.get.assert_called_with(
            '/availableZones',
            endpoint_override='fake/',
            headers={'Accept': 'application/json'},
            params={}
        )

        self.assertIsInstance(rsp[0], az.AvailabilityZone)
        self.assertEqual(JSON_LIST['regionId'], rsp[0].region_id)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.tests.unit import base
from otcextensions.sdk.apig.v2 import apigroup


EXAMPLE_API_GROUP = {
    'id': 'group-12345',
    'gateway_id': 'gateway-67890',
    'name': 'Test API Group',
    'remark': 'This is a test API group',
    'roma_app_id': 'app-54321',
    'version': 'v1',
    'status': 1,
    'sl_domain': 'api.example.com',
    'register_time': '2025-02-07T12:00:00Z',
    'update_time': '2025-02-07T12:30:00Z',
    'on_sell_status': 1,
    'url_domains': [
        {
            'id': 'domain-1',
            'domain': 'example.com',
            'cname_status': 1,
            'ssl_id': 'ssl-123',
            'ssl_name': 'SSL Cert',
            'min_ssl_version': 'TLSv1.2',
            'verified_client_certificate_enabled': True,
            'is_has_trusted_root_ca': False
        }
    ],
    'sl_domain_access_enabled': True,
    'sl_domains': ['sl1.example.com', 'sl2.example.com'],
    'call_limits': 1000,
    'time_interval': 60,
    'time_unit': 'second',
    'is_default': False,
    'roma_app_name': 'Test App'
}


class TestApiGroup(base.TestCase):

    def test_basic(self):
        sot = apigroup.ApiGroup()
        self.assertEqual('/apigw/instances/%(gateway_id)s/api-groups',
                         sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_fetch)
        self.assertEqual('groups', sot.resources_key)

    def test_make_it(self):
        sot = apigroup.ApiGroup(**EXAMPLE_API_GROUP)
        self.assertEqual(EXAMPLE_API_GROUP['id'], sot.id)
        self.assertEqual(EXAMPLE_API_GROUP['gateway_id'], sot.gateway_id)
        self.assertEqual(EXAMPLE_API_GROUP['name'], sot.name)
        self.assertEqual(EXAMPLE_API_GROUP['remark'], sot.remark)
        self.assertEqual(EXAMPLE_API_GROUP['roma_app_id'], sot.roma_app_id)
        self.assertEqual(EXAMPLE_API_GROUP['version'], sot.version)
        self.assertEqual(EXAMPLE_API_GROUP['status'], sot.status)
        self.assertEqual(EXAMPLE_API_GROUP['sl_domain'], sot.sl_domain)
        self.assertEqual(EXAMPLE_API_GROUP['register_time'], sot.register_time)
        self.assertEqual(EXAMPLE_API_GROUP['update_time'], sot.update_time)
        self.assertEqual(EXAMPLE_API_GROUP['on_sell_status'],
                         sot.on_sell_status)
        self.assertEqual(EXAMPLE_API_GROUP['sl_domain_access_enabled'],
                         sot.sl_domain_access_enabled)
        self.assertEqual(EXAMPLE_API_GROUP['sl_domains'], sot.sl_domains)
        self.assertEqual(EXAMPLE_API_GROUP['call_limits'], sot.call_limits)
        self.assertEqual(EXAMPLE_API_GROUP['time_interval'], sot.time_interval)
        self.assertEqual(EXAMPLE_API_GROUP['time_unit'], sot.time_unit)
        self.assertEqual(EXAMPLE_API_GROUP['is_default'], sot.is_default)
        self.assertEqual(EXAMPLE_API_GROUP['roma_app_name'], sot.roma_app_name)

        self.assertEqual(len(EXAMPLE_API_GROUP['url_domains']),
                         len(sot.url_domains))
        self.assertEqual(
            EXAMPLE_API_GROUP['url_domains'][0]['id'],
            sot.url_domains[0].id)
        self.assertEqual(
            EXAMPLE_API_GROUP['url_domains'][0]['domain'],
            sot.url_domains[0].domain)
        self.assertEqual(
            EXAMPLE_API_GROUP['url_domains'][0]['cname_status'],
            sot.url_domains[0].cname_status)
        self.assertEqual(
            EXAMPLE_API_GROUP['url_domains'][0]['ssl_id'],
            sot.url_domains[0].ssl_id)
        self.assertEqual(
            EXAMPLE_API_GROUP['url_domains'][0]['ssl_name'],
            sot.url_domains[0].ssl_name)
        self.assertEqual(
            EXAMPLE_API_GROUP['url_domains'][0]['min_ssl_version'],
            sot.url_domains[0].min_ssl_version)
        self.assertEqual(
            EXAMPLE_API_GROUP['url_domains'][0]
            ['verified_client_certificate_enabled'],
            sot.url_domains[0].verified_client_certificate_enabled)
        self.assertEqual(
            EXAMPLE_API_GROUP['url_domains'][0]['is_has_trusted_root_ca'],
            sot.url_domains[0].is_has_trusted_root_ca)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from keystoneauth1 import adapter

import mock

from openstack.tests.unit import base
from otcextensions.sdk.dis.v2 import checkpoint

EXAMPLE = {
    "sequence_number": "newstram",
    "metadata": ""
}


class TestCheckpoint(base.TestCase):
    def setUp(self):
        super(TestCheckpoint, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)

    def test_basic(self):
        sot = checkpoint.Checkpoint()
        self.assertEqual(None, sot.resource_key)
        self.assertEqual(None, sot.resources_key)
        path = '/checkpoints'
        self.assertEqual(path, sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_delete)
        self.assertFalse(sot.allow_list)
        self.assertFalse(sot.allow_commit)
        self.assertFalse(sot.allow_patch)

    def test_make_it(self):
        sot = checkpoint.Checkpoint(**EXAMPLE)
        updated_sot_attrs = ()
        for key, value in EXAMPLE.items():
            if key not in updated_sot_attrs:
                self.assertEqual(getattr(sot, key), value)

    def test_checkpoint_get(self):
        sot = checkpoint.Checkpoint()
        params = {
            'stream_name': 'test-stream',
            'partition_id': 'test-partition-id',
        }
        response = mock.Mock()
        response.status_code = 200
        response.headers = {}
        response.json.return_value = {
            "sequence_number": "newstram",
            "metadata": ""
        }
        self.sess.get.return_value = response

        rt = sot.get_checkpoint(self.sess, **params)
        self.sess.get.assert_called_with(sot.base_path, params=params)
        self.assertIsNotNone(rt)

    def test_checkpoint_delete(self):
        sot = checkpoint.Checkpoint()
        params = {
            'stream_name': 'test-stream',
            'partition_id': 'test-partition-id',
        }
        response = mock.Mock()
        response.status_code = 204
        response.headers = {}
        response.text.return_value = None
        response.json.return_value = None
        self.sess.delete.return_value = response

        rt = sot.delete_checkpoint(self.sess, **params)
        self.sess.delete.assert_called_with(sot.base_path, params=params)
        self.assertIsNotNone(rt)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.unit import base

from otcextensions.sdk.smn.v2 import subscription


EXAMPLE = {
    "topic_urn": "urn:smn:regionId:762bdb3251034f268af0e..",
    "protocol": "sms",
    "subscription_urn": "urn:smn:regionId:762bdb3251034f268af0e..",
    "owner": "762bdb3251034f268af0e395c53ea09b",
    "endpoint": "xxxxxxxxxx",
    "remark": "",
    "status": 0
}


class TestSubscription(base.TestCase):

    def test_basic(self):
        sot = subscription.Subscription()
        self.assertEqual('subscriptions', sot.resources_key)
        path = '/notifications/topics/%(topic_urn)s/subscriptions'
        self.assertEqual(path, sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_delete)
        self.assertFalse(sot.allow_fetch)
        self.assertFalse(sot.allow_commit)

    def test_make_it(self):
        sot = subscription.Subscription(**EXAMPLE)
        self.assertEqual(EXAMPLE['subscription_urn'], sot.id)
        self.assertEqual(EXAMPLE['topic_urn'], sot.topic_urn)
        self.assertEqual(EXAMPLE['protocol'], sot.protocol)
        self.assertEqual(EXAMPLE['owner'], sot.owner)
        self.assertEqual(EXAMPLE['endpoint'], sot.endpoint)
        self.assertEqual(EXAMPLE['remark'], sot.remark)
        self.assertEqual(EXAMPLE['status'], sot.status)



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
"""Stop a running Devenv Instance (Notebook)."""
import openstack

openstack.enable_logging(True)
conn = openstack.connect(cloud="otc")

instance_id = "notebook_id"
response = conn.modelartsv1.stop_devenv_instance(instance_id)
print(response)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from otcextensions.sdk.lts.v2 import _proxy
from otcextensions.sdk.lts.v2 import group as _group
from otcextensions.sdk.lts.v2 import stream as _stream

from openstack.tests.unit import test_proxy_base


class TestLtsProxy(test_proxy_base.TestProxyBase):

    def setUp(self):
        super(TestLtsProxy, self).setUp()
        self.proxy = _proxy.Proxy(self.session)


class TestGroup(TestLtsProxy):

    def test_groups(self):
        self.verify_list(self.proxy.groups, _group.Group)

    def test_group_create(self):
        self.verify_create(self.proxy.create_group,
                           _group.Group,
                           method_kwargs={'log_group_name': 'log-group-name',
                                          'ttl_in_days': 5},
                           expected_kwargs={'log_group_name': 'log-group-name',
                                            'ttl_in_days': 5})

    def test_group_delete(self):
        self.verify_delete(self.proxy.delete_group,
                           _group.Group, True)


class TestStream(TestLtsProxy):

    def test_streams(self):
        group = _group.Group(id='id-group')
        self.verify_list(
            self.proxy.streams,
            _stream.Stream,
            method_args=[group],
            expected_kwargs={'log_group_id': group.id},
            expected_args=[]
        )

    def test_stream_create(self):
        fake_stream = {
            'log_stream_id': 'log_stream_id',
        }
        self._verify('otcextensions.sdk.lts.v2.group.Group.create_stream',
                     self.proxy.create_stream,
                     expected_result=_stream.Stream.new(**fake_stream),
                     method_result=_stream.Stream.new(**fake_stream),
                     expected_args=[self.proxy],
                     method_kwargs={'log_group': 'log-group-id',
                                    'log_stream_name': 'log-stream-name'},
                     expected_kwargs={
                         'query': {'log_stream_name': 'log-stream-name'}})

    def test_stream_delete(self):
        group = _group.Group(id='id-group')
        stream = _stream.Stream(id='id-stream')
        self._verify(
            'otcextensions.sdk.lts.v2.group.Group.delete_stream',
            self.proxy.delete_stream,
            method_args=[group, stream, True],
            method_kwargs={},
            expected_args=[self.proxy],
            expected_kwargs={
                'log_stream_id': stream.id,
                'ignore_missing': True}
        )



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
'''CSS ELK cluster v1 action implementations'''
import logging

from osc_lib.command import command
from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


class DownloadCertificate(command.Command):
    _description = _('Download the HTTPS certificate file of the server.')

    def get_parser(self, prog_name):
        parser = super(DownloadCertificate, self).get_parser(prog_name)
        parser.add_argument(
            '--out',
            metavar='<out>',
            help=_(
                'Name of the output file where certificate will be saved. '
                '(Optional)'
            ),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.css
        filename = None
        if parsed_args.out:
            filename = parsed_args.out
        client.download_certificate(filename)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from keystoneauth1 import adapter

import mock

from openstack.tests.unit import base

from otcextensions.sdk.auto_scaling.v1 import config

EXAMPLE = {
    'tenant': 'ce061903a53545dcaddb300093b477d2',
    'scaling_configuration_id': '6afe46f9-7d3d-4046-8748-3b2a1085ad86',
    'scaling_configuration_name': 'config_name_1',
    'instance_config': {
        'disk': [
            {
                'size': 40,
                'volume_type': 'SATA',
                'disk_type': 'SYS'
            },
            {
                'size': 100,
                'volume_type': 'SATA',
                'disk_type': 'DATA'
            }
        ],
        'adminPass': '***',
        'personality': None,
        'instance_name': None,
        'instance_id': None,
        'flavorRef': '103',
        'imageRef': '37ca2b35-6fc7-47ab-93c7-900324809c5c',
        'key_name': 'keypair01',
        'public_ip': None,
        'user_data': None,
        'metadata': {}
    },
    'create_time': '2015-07-23T01:04:07Z'
}

EXAMPLE_LIST = {
    'limit': 20,
    'total_number': 2,
    'start_number': 0,
    'scaling_configurations': [
        {
            'tenant_id': 'ce061903a53545dcaddb300093b477d2',
            'status': 'STANDBY',
            'scaling_configuration_id': '6afe46f9-7d3d-4046-8748-3b2a1085ad86',
            'scaling_configuration_name': 'config_name_1',
            'instance_config': {
                'disk': [
                    {
                        'size': 40,
                        'volume_type': 'SATA',
                        'disk_type': 'SYS'
                    },
                    {
                        'size': 100,
                        'volume_type': 'SATA',
                        'disk_type': 'DATA'
                    }
                ],
                'adminPass': '***',
                'personality': None,
                'instance_name': None,
                'instance_id': None,
                'flavorRef': '103',
                'imageRef': '37ca2b35-6fc7-47ab-93c7-900324809c5c',
                'key_name': 'keypair02',
                'public_ip': None,
                'user_data': None,
                'metadate': {}
            },
            'create_time': '2015-07-23T01:04:07Z'
        },
        {
            'tenant_id': 'ce061903a53545dcaddb300093b477d2',
            'status': 'ACTIVE',
            'scaling_configuration_id': '24a8c5f3-c713-4aba-ac29-c17101009e5d',
            'scaling_configuration_name': 'config_name_2',
            'instance_config': {
                'disk': [
                    {
                        'size': 40,
                        'volume_type': 'SATA',
                        'disk_type': 'SYS'
                    }
                ],
                'adminPass': '***',
                'personality': None,
                'instance_name': None,
                'instance_id': None,
                'flavorRef': '103',
                'imageRef': '37ca2b35-6fc7-47ab-93c7-900324809c5c',
                'key_name': 'keypair01',
                'public_ip': None,
                'user_data': None,
                'metadata': {}
            },
            'create_time': '2015-07-22T01:08:41Z'
        }
    ]
}


class TestConfig(base.TestCase):

    def setUp(self):
        super(TestConfig, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.get = mock.Mock()
        self.sess.post = mock.Mock()
        self.sess.delete = mock.Mock()
        self.sess.put = mock.Mock()
        self.sess.get_project_id = mock.Mock()
        self.sot = config.Config(**EXAMPLE)

    def test_basic(self):
        sot = config.Config()
        self.assertEqual('scaling_configurations', sot.resources_key)
        self.assertEqual('/scaling_configuration', sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertFalse(sot.allow_commit)
        self.assertTrue(sot.allow_delete)

    def test_make_it(self):
        sot = config.Config(**EXAMPLE)
        self.assertEqual(EXAMPLE['scaling_configuration_id'], sot.id)
        self.assertEqual(EXAMPLE['scaling_configuration_name'], sot.name)
        self.assertEqual(EXAMPLE['create_time'], sot.create_time)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
'''CSS ELK cluster v1 action implementations'''
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.i18n import _

LOG = logging.getLogger(__name__)

CSS_ENGINE_VERSIONS = ('7.6.2', '7.9.3', '7.10.2')

CSS_NODE_TYPES = (
    'ess',
    'ess-master',
    'ess-client',
    'ess-cold',
)


class ListFlavors(command.Lister):
    _description = _('List Supported Flavors for a Cluster Version.')
    columns = (
        'Id',
        'Name',
        'Version',
        'Type',
        'Availability Zones',
        'Disk Range',
        'vCPUs',
        'RAM',
    )

    def get_parser(self, prog_name):
        parser = super(ListFlavors, self).get_parser(prog_name)

        parser.add_argument(
            '--datastore-version',
            metavar='<datastore_version>',
            type=lambda s: s.lower(),
            choices=CSS_ENGINE_VERSIONS,
            help=_(
                'CSS Cluster Engine Versions. Supported Versions: '
                '{' + ', '.join(CSS_ENGINE_VERSIONS) + '}'
            ),
        )
        parser.add_argument(
            '--node-type',
            metavar='<node_type>',
            type=lambda s: s.lower(),
            choices=CSS_NODE_TYPES,
            help=_(
                'Specify Cluster Node Type. Supported Types: '
                '{' + ', '.join(CSS_NODE_TYPES) + '}'
            ),
        )

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.css
        data = client.flavors()

        node_type = parsed_args.node_type
        datastore_version = parsed_args.datastore_version

        return (
            self.columns,
            (
                utils.get_item_properties(s, self.columns)
                for s in data
                if (not node_type or s.type == node_type)
                and (not datastore_version or s.version == datastore_version)
            ),
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource
from openstack import exceptions


class GatewayFeatures(resource.Resource):
    base_path = '/apigw/instances/%(gateway_id)s/features'
    resources_key = 'features'

    allow_list = True
    allow_create = True

    _query_mapping = resource.QueryParameters('limit', 'offset')

    # Properties
    gateway_id = resource.URI('gateway_id')
    # Feature name.
    name = resource.Body('name', type=str)
    # Indicates whether to enable the feature.
    enable = resource.Body('enable', type=bool)
    # Parameter configuration.
    config = resource.Body('config', type=str)

    # Attributes
    # Feature update time.
    updated_at = resource.Body('update_time')

    def _supported_features(self, session, gateway, **query):
        """List supported Gateway features.
        """
        query_params = self._query_mapping._transpose(query, GatewayFeatures)
        url = f'/apigw/instances/{gateway.id}/instance-features'
        response = session.get(url, params=query_params.copy())
        exceptions.raise_from_response(response)
        data = response.json()
        return data["features"]



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import json
import uuid

import openstack

from otcextensions.tests.functional.sdk.dns import TestDns

_logger = openstack._log.setup_logging('openstack')


class TestZone(TestDns):
    uuid_v4 = uuid.uuid4().hex[:8]
    public_zone_alias = uuid_v4 + 'dns.sdk-test-zone-public.com.'
    private_zone_alias = uuid_v4 + 'dns.sdk-test-zone-private.com.'
    networks = []

    def setUp(self):
        super(TestZone, self).setUp()

    def tearDown(self):
        if self.zone:
            try:
                self.client.delete_zone(self.zone)
                self.client.wait_for_delete_zone(self.zone)
            except openstack.exceptions.SDKException as e:
                _logger.warning('Got exception during clearing resources %s'
                                % e.message)
        super(TestZone, self).tearDown()

    def _create_zone(self, zone_name=None, router_id=None, zone_type='public'):
        if zone_type != 'public' and router_id:
            try:
                self.zone = self.client.create_zone(
                    name=zone_name,
                    router={'router_id': router_id},
                    zone_type=zone_type
                )
                self.client.wait_for_zone(self.zone)
            except openstack.exceptions.BadRequestException:
                self.zone = self.client.find_zone(zone_name)
            return
        try:
            self.zone = self.client.create_zone(
                name=zone_name
            )
            self.client.wait_for_zone(self.zone)
        except openstack.exceptions.BadRequestException:
            self.zone = self.client.find_zone(zone_name)

    def _create_multiple_networks(self, count=1):
        for i in range(count):
            network = self.create_network()
            self.networks.append(network)

    def test_01_list_zones(self):
        self._create_zone(self.public_zone_alias)
        self.all_zones = list(self.client.zones())
        self.assertGreaterEqual(len(self.all_zones), 0)
        if len(self.all_zones) > 0:
            zone = self.all_zones[0]
            zone = self.client.get_zone(zone=zone.id)
            self.assertIsNotNone(zone)

    def test_02_get_zone(self):
        self._create_zone(self.public_zone_alias)
        zone = self.client.get_zone(self.zone.id)
        self.assertEqual(zone.name, self.public_zone_alias)

    def test_03_find_zone(self):
        self._create_zone(self.public_zone_alias)
        zone = self.client.find_zone(self.zone.name)
        self.assertEqual(zone.name, self.public_zone_alias)

    def test_04_update_public_zone(self):
        self._create_zone(self.public_zone_alias)
        description = 'sdk-test-zone-public-description'
        zone = self.client.update_zone(self.zone.id, description=description)
        self.assertEqual(zone.description, description)

    def test_05_update_private_zone(self):
        self._create_multiple_networks(count=2)
        # create private zone
        self._create_zone(
            zone_name=self.private_zone_alias,
            router_id=self.networks[0]['router_id'],
            zone_type='private'
        )
        description = 'sdk-test-zone-private-description'
        zone = self.client.update_zone(
            self.zone.id,
            description=description
        )
        self.assertEqual(zone.description, description)

    def test_06_add_router_to_private_zone(self):
        self._create_zone(
            zone_name=self.private_zone_alias,
            router_id=self.networks[0]['router_id'],
            zone_type='private'
        )
        zone = self.client.add_router_to_zone(
            self.zone.id,
            router_id=self.networks[1]['router_id']
        )
        self.assertEqual(
            json.loads(zone.text)['router_id'],
            self.networks[1]['router_id']
        )
        self.assertEqual(json.loads(zone.text)['status'], 'PENDING_CREATE')

    def test_07_remove_router_from_private_zone(self):
        self._create_zone(
            zone_name=self.private_zone_alias,
            router_id=self.networks[0]['router_id'],
            zone_type='private'
        )
        zone = self.client.add_router_to_zone(
            self.zone.id,
            router_id=self.networks[1]['router_id']
        )
        zone = self.client.remove_router_from_zone(
            self.zone.id,
            router_id=self.networks[1]['router_id']
        )
        for network in self.networks:
            self.destroy_network(network)
        self.assertEqual(
            json.loads(zone.text)['router_id'],
            self.networks[1]['router_id']
        )
        self.assertEqual(json.loads(zone.text)['status'], 'PENDING_DELETE')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import uuid
import openstack

from otcextensions.tests.functional import base

_logger = openstack._log.setup_logging('openstack')


class TestHost(base.BaseFunctionalTest):

    def setUp(self):
        super(TestHost, self).setUp()
        self.client = self.conn.deh
        res = self.client.create_host(
            name=uuid.uuid4().hex,
            availability_zone='eu-de-01',
            host_type='general',
            quantity=1
        )
        assert len(res.dedicated_host_ids) == 1
        host_id = res.dedicated_host_ids[0]
        self.host = self.client.get_host(host_id)

    def tearDown(self):
        try:
            if self.host.id:
                self.client.delete_host(self.host)
        except openstack.exceptions.SDKException as e:
            _logger.warning('Got exception during clearing resources %s'
                            % e.message)

    def test_list(self):
        self.hosts = list(self.conn.deh.hosts())
        self.assertGreaterEqual(len(self.hosts), 0)
        if len(self.hosts) > 0:
            host = self.hosts[0]
            servers = self.client.servers(host=host.id)
            for server in servers:
                _logger.debug(server)

    def test_host_types(self):
        deh = self.conn.deh

        host_types = list(deh.host_types('eu-de-01'))

        self.assertIsNotNone(host_types)
        _logger.debug(host_types)
